import sys
import typing


def BoolProperty(name: str = "",
                 description: str = "",
                 default=False,
                 options: set = {'ANIMATABLE'},
                 tags: set = {},
                 subtype: str = 'NONE',
                 update=None,
                 get=None,
                 set=None):
    '''Returns a new boolean property definition. 

    :param name: Name used in the user interface. 
    :type name: str
    :param description: Text used for the tooltip and api documentation. 
    :type description: str
    :param options: Enumerator in [‘HIDDEN’, ‘SKIP_SAVE’, ‘ANIMATABLE’, ‘LIBRARY_EDITABLE’, ‘PROPORTIONAL’,’TEXTEDIT_UPDATE’]. 
    :type options: set
    :param tags: Enumerator of tags that are defined by parent class. 
    :type tags: set
    :param subtype: Enumerator in [‘PIXEL’, ‘UNSIGNED’, ‘PERCENTAGE’, ‘FACTOR’, ‘ANGLE’, ‘TIME’, ‘DISTANCE’, ‘NONE’]. 
    :type subtype: str
    :param update: Function to be called when this value is modified, This function must take 2 values (self, context) and return None. Warning there are no safety checks to avoid infinite recursion. 
    :param get: Function to be called when this value is ‘read’, This function must take 1 value (self) and return the value of the property. 
    :param set: Function to be called when this value is ‘written’, This function must take 2 values (self, value) and return None. 
    '''

    pass


def BoolVectorProperty(name: str = "",
                       description: str = "",
                       default: list = (False, False, False),
                       options: set = {'ANIMATABLE'},
                       tags: set = {},
                       subtype: str = 'NONE',
                       size: int = 3,
                       update=None,
                       get=None,
                       set=None):
    '''Returns a new vector boolean property definition. 

    :param name: Name used in the user interface. 
    :type name: str
    :param description: Text used for the tooltip and api documentation. 
    :type description: str
    :param default: sequence of booleans the length of size. 
    :type default: list
    :param options: Enumerator in [‘HIDDEN’, ‘SKIP_SAVE’, ‘ANIMATABLE’, ‘LIBRARY_EDITABLE’, ‘PROPORTIONAL’,’TEXTEDIT_UPDATE’]. 
    :type options: set
    :param tags: Enumerator of tags that are defined by parent class. 
    :type tags: set
    :param subtype: Enumerator in [‘COLOR’, ‘TRANSLATION’, ‘DIRECTION’, ‘VELOCITY’, ‘ACCELERATION’, ‘MATRIX’, ‘EULER’, ‘QUATERNION’, ‘AXISANGLE’, ‘XYZ’, ‘COLOR_GAMMA’, ‘LAYER’, ‘LAYER_MEMBER’, ‘POWER’, ‘NONE’]. 
    :type subtype: str
    :param size: Vector dimensions in [1, 32]. 
    :type size: int
    :param update: Function to be called when this value is modified, This function must take 2 values (self, context) and return None. Warning there are no safety checks to avoid infinite recursion. 
    :param get: Function to be called when this value is ‘read’, This function must take 1 value (self) and return the value of the property. 
    :param set: Function to be called when this value is ‘written’, This function must take 2 values (self, value) and return None. 
    '''

    pass


def CollectionProperty(type=None,
                       name: str = "",
                       description: str = "",
                       options: set = {'ANIMATABLE'},
                       tags: set = {}):
    '''Returns a new collection property definition. 

    :param type: A subclass of bpy.types.PropertyGroup or bpy.types.ID. 
    :param name: Name used in the user interface. 
    :type name: str
    :param description: Text used for the tooltip and api documentation. 
    :type description: str
    :param options: Enumerator in [‘HIDDEN’, ‘SKIP_SAVE’, ‘ANIMATABLE’, ‘LIBRARY_EDITABLE’, ‘PROPORTIONAL’,’TEXTEDIT_UPDATE’]. 
    :type options: set
    :param tags: Enumerator of tags that are defined by parent class. 
    :type tags: set
    '''

    pass


def EnumProperty(items: typing.List[str],
                 name: str = "",
                 description: str = "",
                 default: typing.Set[str] = None,
                 options: set = {'ANIMATABLE'},
                 tags: set = {},
                 update=None,
                 get=None,
                 set=None):
    '''Returns a new enumerator property definition. 

    :param items: sequence of enum items formatted: [(identifier, name, description, icon, number), ...].The first three elements of the tuples are mandatory.identifierThe identifier is used for Python access.nameName for the interace.descriptionUsed for documentation and tooltips.iconAn icon string identifier or integer icon value (e.g. returned by bpy.types.UILayout.icon)numberUnique value used as the identifier for this item (stored in file data). Use when the identifier may need to change. If the ENUM_FLAG option is used, the values are bitmasks and should be powers of two.When an item only contains 4 items they define (identifier, name, description, number).For dynamic values a callback can be passed which returns a list in the same format as the static list. This function must take 2 arguments (self, context), context may be None.There is a known bug with using a callback, Python must keep a reference to the strings returned by the callback or Blender will misbehave or even crash. 
    :type items: typing.List[str]
    :param name: Name used in the user interface. 
    :type name: str
    :param description: Text used for the tooltip and api documentation. 
    :type description: str
    :param default: The default value for this enum, a string from the identifiers used in items. If the ENUM_FLAG option is used this must be a set of such string identifiers instead. WARNING: It shall not be specified (or specified to its default None value) for dynamic enums (i.e. if a callback function is given as items parameter). 
    :type default: typing.Set[str]
    :param options: Enumerator in [‘HIDDEN’, ‘SKIP_SAVE’, ‘ANIMATABLE’, ‘ENUM_FLAG’, ‘LIBRARY_EDITABLE’]. 
    :type options: set
    :param tags: Enumerator of tags that are defined by parent class. 
    :type tags: set
    :param update: Function to be called when this value is modified, This function must take 2 values (self, context) and return None. Warning there are no safety checks to avoid infinite recursion. 
    :param get: Function to be called when this value is ‘read’, This function must take 1 value (self) and return the value of the property. 
    :param set: Function to be called when this value is ‘written’, This function must take 2 values (self, value) and return None. 
    '''

    pass


def FloatProperty(name: str = "",
                  description: str = "",
                  default=0.0,
                  min: float = sys.float_info.min,
                  max: float = sys.float_info.max,
                  soft_min: float = sys.float_info.min,
                  soft_max: float = sys.float_info.max,
                  step: int = 3,
                  precision: int = 2,
                  options: set = {'ANIMATABLE'},
                  tags: set = {},
                  subtype: str = 'NONE',
                  unit: str = 'NONE',
                  update=None,
                  get=None,
                  set=None):
    '''Returns a new float property definition. 

    :param name: Name used in the user interface. 
    :type name: str
    :param description: Text used for the tooltip and api documentation. 
    :type description: str
    :param min: Hard minimum, trying to assign a value below will silently assign this minimum instead. 
    :type min: float
    :param max: Hard maximum, trying to assign a value above will silently assign this maximum instead. 
    :type max: float
    :param soft_min: Soft minimum (>= min), user won’t be able to drag the widget below this value in the UI. 
    :type soft_min: float
    :param soft_max: Soft maximum (<= max), user won’t be able to drag the widget above this value in the UI. 
    :type soft_max: float
    :param step: Step of increment/decrement in UI, in [1, 100], defaults to 3 (WARNING: actual value is /100). 
    :type step: int
    :param precision: Maximum number of decimal digits to display, in [0, 6]. 
    :type precision: int
    :param options: Enumerator in [‘HIDDEN’, ‘SKIP_SAVE’, ‘ANIMATABLE’, ‘LIBRARY_EDITABLE’, ‘PROPORTIONAL’,’TEXTEDIT_UPDATE’]. 
    :type options: set
    :param tags: Enumerator of tags that are defined by parent class. 
    :type tags: set
    :param subtype: Enumerator in [‘PIXEL’, ‘UNSIGNED’, ‘PERCENTAGE’, ‘FACTOR’, ‘ANGLE’, ‘TIME’, ‘DISTANCE’, ‘NONE’]. 
    :type subtype: str
    :param unit: Enumerator in [‘NONE’, ‘LENGTH’, ‘AREA’, ‘VOLUME’, ‘ROTATION’, ‘TIME’, ‘VELOCITY’, ‘ACCELERATION’, ‘MASS’, ‘CAMERA’, ‘POWER’]. 
    :type unit: str
    :param update: Function to be called when this value is modified, This function must take 2 values (self, context) and return None. Warning there are no safety checks to avoid infinite recursion. 
    :param get: Function to be called when this value is ‘read’, This function must take 1 value (self) and return the value of the property. 
    :param set: Function to be called when this value is ‘written’, This function must take 2 values (self, value) and return None. 
    '''

    pass


def FloatVectorProperty(name: str = "",
                        description: str = "",
                        default: list = (0.0, 0.0, 0.0),
                        min: float = sys.float_info.min,
                        max: float = sys.float_info.max,
                        soft_min: float = sys.float_info.min,
                        soft_max: float = sys.float_info.max,
                        step: int = 3,
                        precision: int = 2,
                        options: set = {'ANIMATABLE'},
                        tags: set = {},
                        subtype: str = 'NONE',
                        unit: str = 'NONE',
                        size: int = 3,
                        update=None,
                        get=None,
                        set=None):
    '''Returns a new vector float property definition. 

    :param name: Name used in the user interface. 
    :type name: str
    :param description: Text used for the tooltip and api documentation. 
    :type description: str
    :param default: sequence of floats the length of size. 
    :type default: list
    :param min: Hard minimum, trying to assign a value below will silently assign this minimum instead. 
    :type min: float
    :param max: Hard maximum, trying to assign a value above will silently assign this maximum instead. 
    :type max: float
    :param soft_min: Soft minimum (>= min), user won’t be able to drag the widget below this value in the UI. 
    :type soft_min: float
    :param soft_max: Soft maximum (<= max), user won’t be able to drag the widget above this value in the UI. 
    :type soft_max: float
    :param options: Enumerator in [‘HIDDEN’, ‘SKIP_SAVE’, ‘ANIMATABLE’, ‘LIBRARY_EDITABLE’, ‘PROPORTIONAL’,’TEXTEDIT_UPDATE’]. 
    :type options: set
    :param tags: Enumerator of tags that are defined by parent class. 
    :type tags: set
    :param step: Step of increment/decrement in UI, in [1, 100], defaults to 3 (WARNING: actual value is /100). 
    :type step: int
    :param precision: Maximum number of decimal digits to display, in [0, 6]. 
    :type precision: int
    :param subtype: Enumerator in [‘COLOR’, ‘TRANSLATION’, ‘DIRECTION’, ‘VELOCITY’, ‘ACCELERATION’, ‘MATRIX’, ‘EULER’, ‘QUATERNION’, ‘AXISANGLE’, ‘XYZ’, ‘COLOR_GAMMA’, ‘LAYER’, ‘LAYER_MEMBER’, ‘POWER’, ‘NONE’]. 
    :type subtype: str
    :param unit: Enumerator in [‘NONE’, ‘LENGTH’, ‘AREA’, ‘VOLUME’, ‘ROTATION’, ‘TIME’, ‘VELOCITY’, ‘ACCELERATION’, ‘MASS’, ‘CAMERA’, ‘POWER’]. 
    :type unit: str
    :param size: Vector dimensions in [1, 32]. 
    :type size: int
    :param update: Function to be called when this value is modified, This function must take 2 values (self, context) and return None. Warning there are no safety checks to avoid infinite recursion. 
    :param get: Function to be called when this value is ‘read’, This function must take 1 value (self) and return the value of the property. 
    :param set: Function to be called when this value is ‘written’, This function must take 2 values (self, value) and return None. 
    '''

    pass


def IntProperty(name: str = "",
                description: str = "",
                default=0,
                min: int = -2**31,
                max: int = 2**31 - 1,
                soft_min: int = -2**31,
                soft_max: int = 2**31 - 1,
                step: int = 1,
                options: set = {'ANIMATABLE'},
                tags: set = {},
                subtype: str = 'NONE',
                update=None,
                get=None,
                set=None):
    '''Returns a new int property definition. 

    :param name: Name used in the user interface. 
    :type name: str
    :param description: Text used for the tooltip and api documentation. 
    :type description: str
    :param min: Hard minimum, trying to assign a value below will silently assign this minimum instead. 
    :type min: int
    :param max: Hard maximum, trying to assign a value above will silently assign this maximum instead. 
    :type max: int
    :param soft_max: Soft maximum (<= max), user won’t be able to drag the widget above this value in the UI. 
    :type soft_max: int
    :param soft_min: Soft minimum (>= min), user won’t be able to drag the widget below this value in the UI. 
    :type soft_min: int
    :param step: Step of increment/decrement in UI, in [1, 100], defaults to 1 (WARNING: unused currently!). 
    :type step: int
    :param options: Enumerator in [‘HIDDEN’, ‘SKIP_SAVE’, ‘ANIMATABLE’, ‘LIBRARY_EDITABLE’, ‘PROPORTIONAL’,’TEXTEDIT_UPDATE’]. 
    :type options: set
    :param tags: Enumerator of tags that are defined by parent class. 
    :type tags: set
    :param subtype: Enumerator in [‘PIXEL’, ‘UNSIGNED’, ‘PERCENTAGE’, ‘FACTOR’, ‘ANGLE’, ‘TIME’, ‘DISTANCE’, ‘NONE’]. 
    :type subtype: str
    :param update: Function to be called when this value is modified, This function must take 2 values (self, context) and return None. Warning there are no safety checks to avoid infinite recursion. 
    :param get: Function to be called when this value is ‘read’, This function must take 1 value (self) and return the value of the property. 
    :param set: Function to be called when this value is ‘written’, This function must take 2 values (self, value) and return None. 
    '''

    pass


def IntVectorProperty(name: str = "",
                      description: str = "",
                      default: list = (0, 0, 0),
                      min: int = -2**31,
                      max: int = 2**31 - 1,
                      soft_min: int = -2**31,
                      soft_max: int = 2**31 - 1,
                      step: int = 1,
                      options: set = {'ANIMATABLE'},
                      tags: set = {},
                      subtype: str = 'NONE',
                      size: int = 3,
                      update=None,
                      get=None,
                      set=None):
    '''Returns a new vector int property definition. 

    :param name: Name used in the user interface. 
    :type name: str
    :param description: Text used for the tooltip and api documentation. 
    :type description: str
    :param default: sequence of ints the length of size. 
    :type default: list
    :param min: Hard minimum, trying to assign a value below will silently assign this minimum instead. 
    :type min: int
    :param max: Hard maximum, trying to assign a value above will silently assign this maximum instead. 
    :type max: int
    :param soft_min: Soft minimum (>= min), user won’t be able to drag the widget below this value in the UI. 
    :type soft_min: int
    :param soft_max: Soft maximum (<= max), user won’t be able to drag the widget above this value in the UI. 
    :type soft_max: int
    :param step: Step of increment/decrement in UI, in [1, 100], defaults to 1 (WARNING: unused currently!). 
    :type step: int
    :param options: Enumerator in [‘HIDDEN’, ‘SKIP_SAVE’, ‘ANIMATABLE’, ‘LIBRARY_EDITABLE’, ‘PROPORTIONAL’,’TEXTEDIT_UPDATE’]. 
    :type options: set
    :param tags: Enumerator of tags that are defined by parent class. 
    :type tags: set
    :param subtype: Enumerator in [‘COLOR’, ‘TRANSLATION’, ‘DIRECTION’, ‘VELOCITY’, ‘ACCELERATION’, ‘MATRIX’, ‘EULER’, ‘QUATERNION’, ‘AXISANGLE’, ‘XYZ’, ‘COLOR_GAMMA’, ‘LAYER’, ‘LAYER_MEMBER’, ‘POWER’, ‘NONE’]. 
    :type subtype: str
    :param size: Vector dimensions in [1, 32]. 
    :type size: int
    :param update: Function to be called when this value is modified, This function must take 2 values (self, context) and return None. Warning there are no safety checks to avoid infinite recursion. 
    :param get: Function to be called when this value is ‘read’, This function must take 1 value (self) and return the value of the property. 
    :param set: Function to be called when this value is ‘written’, This function must take 2 values (self, value) and return None. 
    '''

    pass


def PointerProperty(type=None,
                    name: str = "",
                    description: str = "",
                    options: set = {'ANIMATABLE'},
                    tags: set = {},
                    poll=None,
                    update=None):
    '''Returns a new pointer property definition. 

    :param type: A subclass of bpy.types.PropertyGroup or bpy.types.ID. 
    :param name: Name used in the user interface. 
    :type name: str
    :param description: Text used for the tooltip and api documentation. 
    :type description: str
    :param options: Enumerator in [‘HIDDEN’, ‘SKIP_SAVE’, ‘ANIMATABLE’, ‘LIBRARY_EDITABLE’, ‘PROPORTIONAL’,’TEXTEDIT_UPDATE’]. 
    :type options: set
    :param tags: Enumerator of tags that are defined by parent class. 
    :type tags: set
    :param poll: function to be called to determine whether an item is valid for this property. The function must take 2 values (self, object) and return Bool. 
    :param update: Function to be called when this value is modified, This function must take 2 values (self, context) and return None. Warning there are no safety checks to avoid infinite recursion. 
    '''

    pass


def RemoveProperty(cls, attr: str):
    '''Removes a dynamically defined property. 

    :param cls: The class containing the property (must be a positional argument). 
    :param attr: Property name (must be passed as a keyword). 
    :type attr: str
    '''

    pass


def StringProperty(name: str = "",
                   description: str = "",
                   default: str = "",
                   maxlen: int = 0,
                   options: set = {'ANIMATABLE'},
                   tags: set = {},
                   subtype: str = 'NONE',
                   update=None,
                   get=None,
                   set=None):
    '''Returns a new string property definition. 

    :param name: Name used in the user interface. 
    :type name: str
    :param description: Text used for the tooltip and api documentation. 
    :type description: str
    :param default: initializer string. 
    :type default: str
    :param maxlen: maximum length of the string. 
    :type maxlen: int
    :param options: Enumerator in [‘HIDDEN’, ‘SKIP_SAVE’, ‘ANIMATABLE’, ‘LIBRARY_EDITABLE’, ‘PROPORTIONAL’,’TEXTEDIT_UPDATE’]. 
    :type options: set
    :param tags: Enumerator of tags that are defined by parent class. 
    :type tags: set
    :param subtype: Enumerator in [‘FILE_PATH’, ‘DIR_PATH’, ‘FILE_NAME’, ‘BYTE_STRING’, ‘PASSWORD’, ‘NONE’]. 
    :type subtype: str
    :param update: Function to be called when this value is modified, This function must take 2 values (self, context) and return None. Warning there are no safety checks to avoid infinite recursion. 
    :param get: Function to be called when this value is ‘read’, This function must take 1 value (self) and return the value of the property. 
    :param set: Function to be called when this value is ‘written’, This function must take 2 values (self, value) and return None. 
    '''

    pass



import sys
import typing
import bpy


def add_marker(location: float = (0.0, 0.0)):
    '''Place new marker at specified location 

    :param location: Location, Location of marker on frame 
    :type location: float
    '''

    pass


def add_marker_at_click():
    '''Place new marker at the desired (clicked) position 

    '''

    pass


def add_marker_move(CLIP_OT_add_marker=None, TRANSFORM_OT_translate=None):
    '''Add new marker and move it on movie 

    :param CLIP_OT_add_marker: Add Marker, Place new marker at specified location 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def add_marker_slide(CLIP_OT_add_marker=None, TRANSFORM_OT_translate=None):
    '''Add new marker and slide it with mouse until mouse button release 

    :param CLIP_OT_add_marker: Add Marker, Place new marker at specified location 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def apply_solution_scale(distance: float = 0.0):
    '''Apply scale on solution itself to make distance between selected tracks equals to desired 

    :param distance: Distance, Distance between selected tracks 
    :type distance: float
    '''

    pass


def bundles_to_mesh():
    '''Create vertex cloud using coordinates of reconstructed tracks 

    '''

    pass


def camera_preset_add(name: str = "",
                      remove_name: bool = False,
                      remove_active: bool = False,
                      use_focal_length: bool = True):
    '''Add or remove a Tracking Camera Intrinsics Preset 

    :param name: Name, Name of the preset, used to make the path name 
    :type name: str
    :param remove_name: remove_name 
    :type remove_name: bool
    :param remove_active: remove_active 
    :type remove_active: bool
    :param use_focal_length: Include Focal Length, Include focal length into the preset 
    :type use_focal_length: bool
    '''

    pass


def change_frame(frame: int = 0):
    '''Interactively change the current frame number 

    :param frame: Frame 
    :type frame: int
    '''

    pass


def clean_tracks(frames: int = 0,
                 error: float = 0.0,
                 action: typing.Union[int, str] = 'SELECT'):
    '''Clean tracks with high error values or few frames 

    :param frames: Tracked Frames, Effect on tracks which are tracked less than specified amount of frames 
    :type frames: int
    :param error: Reprojection Error, Effect on tracks which have got larger re-projection error 
    :type error: float
    :param action: Action, Cleanup action to executeSELECT Select, Select unclean tracks.DELETE_TRACK Delete Track, Delete unclean tracks.DELETE_SEGMENTS Delete Segments, Delete unclean segments of tracks. 
    :type action: typing.Union[int, str]
    '''

    pass


def clear_solution():
    '''Clear all calculated data 

    '''

    pass


def clear_track_path(action: typing.Union[int, str] = 'REMAINED',
                     clear_active: bool = False):
    '''Clear tracks after/before current position or clear the whole track 

    :param action: Action, Clear action to executeUPTO Clear up-to, Clear path up to current frame.REMAINED Clear remained, Clear path at remaining frames (after current).ALL Clear all, Clear the whole path. 
    :type action: typing.Union[int, str]
    :param clear_active: Clear Active, Clear active track only instead of all selected tracks 
    :type clear_active: bool
    '''

    pass


def constraint_to_fcurve():
    '''Create F-Curves for object which will copy object’s movement caused by this constraint 

    '''

    pass


def copy_tracks():
    '''Copy selected tracks to clipboard 

    '''

    pass


def create_plane_track():
    '''Create new plane track out of selected point tracks 

    '''

    pass


def cursor_set(location: float = (0.0, 0.0)):
    '''Set 2D cursor location 

    :param location: Location, Cursor location in normalized clip coordinates 
    :type location: float
    '''

    pass


def delete_marker():
    '''Delete marker for current frame from selected tracks 

    '''

    pass


def delete_proxy():
    '''Delete movie clip proxy files from the hard drive 

    '''

    pass


def delete_track():
    '''Delete selected tracks 

    '''

    pass


def detect_features(placement: typing.Union[int, str] = 'FRAME',
                    margin: int = 16,
                    threshold: float = 0.5,
                    min_distance: int = 120):
    '''Automatically detect features and place markers to track 

    :param placement: Placement, Placement for detected featuresFRAME Whole Frame, Place markers across the whole frame.INSIDE_GPENCIL Inside Grease Pencil, Place markers only inside areas outlined with Grease Pencil.OUTSIDE_GPENCIL Outside Grease Pencil, Place markers only outside areas outlined with Grease Pencil. 
    :type placement: typing.Union[int, str]
    :param margin: Margin, Only features further than margin pixels from the image edges are considered 
    :type margin: int
    :param threshold: Threshold, Threshold level to consider feature good enough for tracking 
    :type threshold: float
    :param min_distance: Distance, Minimal distance accepted between two features 
    :type min_distance: int
    '''

    pass


def disable_markers(action: typing.Union[int, str] = 'DISABLE'):
    '''Disable/enable selected markers 

    :param action: Action, Disable action to executeDISABLE Disable, Disable selected markers.ENABLE Enable, Enable selected markers.TOGGLE Toggle, Toggle disabled flag for selected markers. 
    :type action: typing.Union[int, str]
    '''

    pass


def dopesheet_select_channel(location: float = (0.0, 0.0),
                             extend: bool = False):
    '''Select movie tracking channel 

    :param location: Location, Mouse location to select channel 
    :type location: float
    :param extend: Extend, Extend selection rather than clearing the existing selection 
    :type extend: bool
    '''

    pass


def dopesheet_view_all():
    '''Reset viewable area to show full keyframe range 

    '''

    pass


def filter_tracks(track_threshold: float = 5.0):
    '''Filter tracks which has weirdly looking spikes in motion curves 

    :param track_threshold: Track Threshold, Filter Threshold to select problematic tracks 
    :type track_threshold: float
    '''

    pass


def frame_jump(position: typing.Union[int, str] = 'PATHSTART'):
    '''Jump to special frame 

    :param position: Position, Position to jump toPATHSTART Path Start, Jump to start of current path.PATHEND Path End, Jump to end of current path.FAILEDPREV Previous Failed, Jump to previous failed frame.FAILNEXT Next Failed, Jump to next failed frame. 
    :type position: typing.Union[int, str]
    '''

    pass


def graph_center_current_frame():
    '''Scroll view so current frame would be centered 

    '''

    pass


def graph_delete_curve():
    '''Delete track corresponding to the selected curve 

    '''

    pass


def graph_delete_knot():
    '''Delete curve knots 

    '''

    pass


def graph_disable_markers(action: typing.Union[int, str] = 'DISABLE'):
    '''Disable/enable selected markers 

    :param action: Action, Disable action to executeDISABLE Disable, Disable selected markers.ENABLE Enable, Enable selected markers.TOGGLE Toggle, Toggle disabled flag for selected markers. 
    :type action: typing.Union[int, str]
    '''

    pass


def graph_select(location: float = (0.0, 0.0), extend: bool = False):
    '''Select graph curves 

    :param location: Location, Mouse location to select nearest entity 
    :type location: float
    :param extend: Extend, Extend selection rather than clearing the existing selection 
    :type extend: bool
    '''

    pass


def graph_select_all_markers(action: typing.Union[int, str] = 'TOGGLE'):
    '''Change selection of all markers of active track 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def graph_select_box(xmin: int = 0,
                     xmax: int = 0,
                     ymin: int = 0,
                     ymax: int = 0,
                     wait_for_input: bool = True,
                     deselect: bool = False,
                     extend: bool = True):
    '''Select curve points using box selection 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param deselect: Deselect, Deselect rather than select items 
    :type deselect: bool
    :param extend: Extend, Extend selection instead of deselecting everything first 
    :type extend: bool
    '''

    pass


def graph_view_all():
    '''View all curves in editor 

    '''

    pass


def hide_tracks(unselected: bool = False):
    '''Hide selected tracks 

    :param unselected: Unselected, Hide unselected tracks 
    :type unselected: bool
    '''

    pass


def hide_tracks_clear():
    '''Clear hide selected tracks 

    '''

    pass


def join_tracks():
    '''Join selected tracks 

    '''

    pass


def keyframe_delete():
    '''Delete a keyframe from selected tracks at current frame 

    '''

    pass


def keyframe_insert():
    '''Insert a keyframe to selected tracks at current frame 

    '''

    pass


def lock_tracks(action: typing.Union[int, str] = 'LOCK'):
    '''Lock/unlock selected tracks 

    :param action: Action, Lock action to executeLOCK Lock, Lock selected tracks.UNLOCK Unlock, Unlock selected tracks.TOGGLE Toggle, Toggle locked flag for selected tracks. 
    :type action: typing.Union[int, str]
    '''

    pass


def mode_set(mode: typing.Union[int, str] = 'TRACKING'):
    '''Set the clip interaction mode 

    :param mode: ModeTRACKING Tracking, Show tracking and solving tools.MASK Mask, Show mask editing tools. 
    :type mode: typing.Union[int, str]
    '''

    pass


def open(directory: str = "",
         files: typing.List['bpy.types.OperatorFileListElement'] = None,
         filter_blender: bool = False,
         filter_backup: bool = False,
         filter_image: bool = True,
         filter_movie: bool = True,
         filter_python: bool = False,
         filter_font: bool = False,
         filter_sound: bool = False,
         filter_text: bool = False,
         filter_btx: bool = False,
         filter_collada: bool = False,
         filter_alembic: bool = False,
         filter_folder: bool = True,
         filter_blenlib: bool = False,
         filemode: int = 9,
         relative_path: bool = True,
         show_multiview: bool = False,
         use_multiview: bool = False,
         display_type: typing.Union[int, str] = 'DEFAULT',
         sort_method: typing.Union[int, str] = 'FILE_SORT_ALPHA'):
    '''Load a sequence of frames or a movie file 

    :param directory: Directory, Directory of the file 
    :type directory: str
    :param files: Files 
    :type files: typing.List['bpy.types.OperatorFileListElement']
    :param filter_blender: Filter .blend files 
    :type filter_blender: bool
    :param filter_backup: Filter .blend files 
    :type filter_backup: bool
    :param filter_image: Filter image files 
    :type filter_image: bool
    :param filter_movie: Filter movie files 
    :type filter_movie: bool
    :param filter_python: Filter python files 
    :type filter_python: bool
    :param filter_font: Filter font files 
    :type filter_font: bool
    :param filter_sound: Filter sound files 
    :type filter_sound: bool
    :param filter_text: Filter text files 
    :type filter_text: bool
    :param filter_btx: Filter btx files 
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files 
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files 
    :type filter_alembic: bool
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs 
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file 
    :type filemode: int
    :param relative_path: Relative Path, Select the file relative to the blend file 
    :type relative_path: bool
    :param show_multiview: Enable Multi-View 
    :type show_multiview: bool
    :param use_multiview: Use Multi-View 
    :type use_multiview: bool
    :param display_type: Display TypeDEFAULT Default, Automatically determine display type for files.LIST_SHORT Short List, Display files as short list.LIST_LONG Long List, Display files as a detailed list.THUMBNAIL Thumbnails, Display files as thumbnails. 
    :type display_type: typing.Union[int, str]
    :param sort_method: File sorting modeFILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically.FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type.FILE_SORT_TIME Sort by time, Sort files by modification time.FILE_SORT_SIZE Sort by size, Sort files by size. 
    :type sort_method: typing.Union[int, str]
    '''

    pass


def paste_tracks():
    '''Paste tracks from clipboard 

    '''

    pass


def prefetch():
    '''Prefetch frames from disk for faster playback/tracking 

    '''

    pass


def rebuild_proxy():
    '''Rebuild all selected proxies and timecode indices in the background 

    '''

    pass


def refine_markers(backwards: bool = False):
    '''Refine selected markers positions by running the tracker from track’s reference to current frame 

    :param backwards: Backwards, Do backwards tracking 
    :type backwards: bool
    '''

    pass


def reload():
    '''Reload clip 

    '''

    pass


def select(extend: bool = False,
           deselect_all: bool = False,
           location: float = (0.0, 0.0)):
    '''Select tracking markers 

    :param extend: Extend, Extend selection rather than clearing the existing selection 
    :type extend: bool
    :param deselect_all: Deselect On Nothing, Deselect all when nothing under the cursor 
    :type deselect_all: bool
    :param location: Location, Mouse location in normalized coordinates, 0.0 to 1.0 is within the image bounds 
    :type location: float
    '''

    pass


def select_all(action: typing.Union[int, str] = 'TOGGLE'):
    '''Change selection of all tracking markers 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_box(xmin: int = 0,
               xmax: int = 0,
               ymin: int = 0,
               ymax: int = 0,
               wait_for_input: bool = True,
               mode: typing.Union[int, str] = 'SET'):
    '''Select markers using box selection 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    '''

    pass


def select_circle(x: int = 0,
                  y: int = 0,
                  radius: int = 25,
                  wait_for_input: bool = True,
                  mode: typing.Union[int, str] = 'SET'):
    '''Select markers using circle selection 

    :param x: X 
    :type x: int
    :param y: Y 
    :type y: int
    :param radius: Radius 
    :type radius: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    '''

    pass


def select_grouped(group: typing.Union[int, str] = 'ESTIMATED'):
    '''Select all tracks from specified group 

    :param group: Action, Clear action to executeKEYFRAMED Keyframed tracks, Select all keyframed tracks.ESTIMATED Estimated tracks, Select all estimated tracks.TRACKED Tracked tracks, Select all tracked tracks.LOCKED Locked tracks, Select all locked tracks.DISABLED Disabled tracks, Select all disabled tracks.COLOR Tracks with same color, Select all tracks with same color as active track.FAILED Failed Tracks, Select all tracks which failed to be reconstructed. 
    :type group: typing.Union[int, str]
    '''

    pass


def select_lasso(path: typing.List['bpy.types.OperatorMousePath'] = None,
                 mode: typing.Union[int, str] = 'SET'):
    '''Select markers using lasso selection 

    :param path: Path 
    :type path: typing.List['bpy.types.OperatorMousePath']
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    '''

    pass


def set_active_clip():
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    '''

    pass


def set_axis(axis: typing.Union[int, str] = 'X'):
    '''Set direction of scene axis rotating camera (or its parent if present) and assume selected track lies on real axis, joining it with the origin 

    :param axis: Axis, Axis to use to align bundle alongX X, Align bundle align X axis.Y Y, Align bundle align Y axis. 
    :type axis: typing.Union[int, str]
    '''

    pass


def set_center_principal():
    '''Set optical center to center of footage 

    '''

    pass


def set_origin(use_median: bool = False):
    '''Set active marker as origin by moving camera (or its parent if present) in 3D space 

    :param use_median: Use Median, Set origin to median point of selected bundles 
    :type use_median: bool
    '''

    pass


def set_plane(plane: typing.Union[int, str] = 'FLOOR'):
    '''Set plane based on 3 selected bundles by moving camera (or its parent if present) in 3D space 

    :param plane: Plane, Plane to be used for orientationFLOOR Floor, Set floor plane.WALL Wall, Set wall plane. 
    :type plane: typing.Union[int, str]
    '''

    pass


def set_scale(distance: float = 0.0):
    '''Set scale of scene by scaling camera (or its parent if present) 

    :param distance: Distance, Distance between selected tracks 
    :type distance: float
    '''

    pass


def set_scene_frames():
    '''Set scene’s start and end frame to match clip’s start frame and length 

    '''

    pass


def set_solution_scale(distance: float = 0.0):
    '''Set object solution scale using distance between two selected tracks 

    :param distance: Distance, Distance between selected tracks 
    :type distance: float
    '''

    pass


def set_solver_keyframe(keyframe: typing.Union[int, str] = 'KEYFRAME_A'):
    '''Set keyframe used by solver 

    :param keyframe: Keyframe, Keyframe to set 
    :type keyframe: typing.Union[int, str]
    '''

    pass


def set_viewport_background():
    '''Set current movie clip as a camera background in 3D view-port (works only when a 3D view-port is visible) 

    '''

    pass


def setup_tracking_scene():
    '''Prepare scene for compositing 3D objects into this footage 

    '''

    pass


def slide_marker(offset: float = (0.0, 0.0)):
    '''Slide marker areas 

    :param offset: Offset, Offset in floating point units, 1.0 is the width and height of the image 
    :type offset: float
    '''

    pass


def slide_plane_marker():
    '''Slide plane marker areas 

    '''

    pass


def solve_camera():
    '''Solve camera motion from tracks 

    '''

    pass


def stabilize_2d_add():
    '''Add selected tracks to 2D translation stabilization 

    '''

    pass


def stabilize_2d_remove():
    '''Remove selected track from translation stabilization 

    '''

    pass


def stabilize_2d_rotation_add():
    '''Add selected tracks to 2D rotation stabilization 

    '''

    pass


def stabilize_2d_rotation_remove():
    '''Remove selected track from rotation stabilization 

    '''

    pass


def stabilize_2d_rotation_select():
    '''Select tracks which are used for rotation stabilization 

    '''

    pass


def stabilize_2d_select():
    '''Select tracks which are used for translation stabilization 

    '''

    pass


def track_color_preset_add(name: str = "",
                           remove_name: bool = False,
                           remove_active: bool = False):
    '''Add or remove a Clip Track Color Preset 

    :param name: Name, Name of the preset, used to make the path name 
    :type name: str
    :param remove_name: remove_name 
    :type remove_name: bool
    :param remove_active: remove_active 
    :type remove_active: bool
    '''

    pass


def track_copy_color():
    '''Copy color to all selected tracks 

    '''

    pass


def track_markers(backwards: bool = False, sequence: bool = False):
    '''Track selected markers 

    :param backwards: Backwards, Do backwards tracking 
    :type backwards: bool
    :param sequence: Track Sequence, Track marker during image sequence rather than single image 
    :type sequence: bool
    '''

    pass


def track_settings_as_default():
    '''Copy tracking settings from active track to default settings 

    '''

    pass


def track_settings_to_track():
    '''Copy tracking settings from active track to selected tracks 

    '''

    pass


def track_to_empty():
    '''Create an Empty object which will be copying movement of active track 

    '''

    pass


def tracking_object_new():
    '''Add new object for tracking 

    '''

    pass


def tracking_object_remove():
    '''Remove object for tracking 

    '''

    pass


def tracking_settings_preset_add(name: str = "",
                                 remove_name: bool = False,
                                 remove_active: bool = False):
    '''Add or remove a motion tracking settings preset 

    :param name: Name, Name of the preset, used to make the path name 
    :type name: str
    :param remove_name: remove_name 
    :type remove_name: bool
    :param remove_active: remove_active 
    :type remove_active: bool
    '''

    pass


def view_all(fit_view: bool = False):
    '''View whole image with markers 

    :param fit_view: Fit View, Fit frame to the viewport 
    :type fit_view: bool
    '''

    pass


def view_ndof():
    '''Use a 3D mouse device to pan/zoom the view 

    '''

    pass


def view_pan(offset: float = (0.0, 0.0)):
    '''Pan the view 

    :param offset: Offset, Offset in floating point units, 1.0 is the width and height of the image 
    :type offset: float
    '''

    pass


def view_selected():
    '''View all selected elements 

    '''

    pass


def view_zoom(factor: float = 0.0, use_cursor_init: bool = True):
    '''Zoom in/out the view 

    :param factor: Factor, Zoom factor, values higher than 1.0 zoom in, lower values zoom out 
    :type factor: float
    :param use_cursor_init: Cursor Init, Use initial cursor position 
    :type use_cursor_init: bool
    '''

    pass


def view_zoom_in(location: float = (0.0, 0.0)):
    '''Zoom in the view 

    :param location: Location, Cursor location in screen coordinates 
    :type location: float
    '''

    pass


def view_zoom_out(location: float = (0.0, 0.0)):
    '''Zoom out the view 

    :param location: Location, Cursor location in normalized (0.0-1.0) coordinates 
    :type location: float
    '''

    pass


def view_zoom_ratio(ratio: float = 0.0):
    '''Set the zoom ratio (based on clip size) 

    :param ratio: Ratio, Zoom ratio, 1.0 is 1:1, higher is zoomed in, lower is zoomed out 
    :type ratio: float
    '''

    pass



import sys
import typing


def new():
    '''Add a new texture 

    '''

    pass


def slot_copy():
    '''Copy the material texture settings and nodes 

    '''

    pass


def slot_move(type: typing.Union[int, str] = 'UP'):
    '''Move texture slots up and down 

    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def slot_paste():
    '''Copy the texture settings and nodes 

    '''

    pass



import sys
import typing


class KDTree:
    '''KdTree(size) -> new kd-tree initialized to hold size items. '''

    def balance(self, ):
        '''Balance the tree. 

        '''
        pass

    def find(self, co: float, filter=None):
        '''Find nearest point to co. 

        :param co: 3d coordinates. 
        :type co: float
        :param filter: function which takes an index and returns True for indices to include in the search. 
        :type filter: 
        :return:  Returns (Vector, index, distance). 
        '''
        pass

    def find_n(self, co: float, n: int) -> list:
        '''Find nearest n points to co. 

        :param co: 3d coordinates. 
        :type co: float
        :param n: Number of points to find. 
        :type n: int
        :rtype: list
        :return:  Returns a list of tuples (Vector, index, distance). 
        '''
        pass

    def find_range(self, co: float, radius: float) -> list:
        '''Find all points within radius of co. 

        :param co: 3d coordinates. 
        :type co: float
        :param radius: Distance to search for points. 
        :type radius: float
        :rtype: list
        :return:  Returns a list of tuples (Vector, index, distance). 
        '''
        pass

    def insert(self, co: float, index: int):
        '''Insert a point into the KDTree. 

        :param co: Point 3d position. 
        :type co: float
        :param index: The index of the point. 
        :type index: int
        '''
        pass

    def __init__(self, size):
        '''

        '''
        pass



import sys
import typing


def add():
    '''Add a new workspace by duplicating the current one or appending one from the user configuration 

    '''

    pass


def append_activate(idname: str = "", filepath: str = ""):
    '''Append a workspace and make it the active one in the current window 

    :param idname: Identifier, Name of the workspace to append and activate 
    :type idname: str
    :param filepath: Filepath, Path to the library 
    :type filepath: str
    '''

    pass


def delete():
    '''Delete the active workspace 

    '''

    pass


def duplicate():
    '''Add a new workspace 

    '''

    pass


def reorder_to_back():
    '''Reorder workspace to be first in the list 

    '''

    pass


def reorder_to_front():
    '''Reorder workspace to be first in the list 

    '''

    pass



import sys
import typing
import bpy
import mathutils


class AddObjectHelper:
    def align_update_callback(self, _context):
        '''

        '''
        pass


def add_object_align_init(
        context: 'bpy.types.Context',
        operator: 'bpy.types.Operator') -> 'mathutils.Matrix':
    '''Return a matrix using the operator settings and view context. 

    :param context: The context to use. 
    :type context: 'bpy.types.Context'
    :param operator: The operator, checked for location and rotation properties. 
    :type operator: 'bpy.types.Operator'
    :return:  the matrix from the context and settings. 
    '''

    pass


def object_add_grid_scale(context):
    '''Return scale which should be applied on object data to align it to grid scale 

    '''

    pass


def object_add_grid_scale_apply_operator(operator, context):
    '''Scale an operators distance values by the grid size. 

    '''

    pass


def object_data_add(context: 'bpy.types.Context',
                    obdata: 'bpy.data',
                    operator: 'bpy.types.Operator' = None,
                    name: str = None) -> 'bpy.types.Object':
    '''Add an object using the view context and preference to initialize the location, rotation and layer. 

    :param context: The context to use. 
    :type context: 'bpy.types.Context'
    :param obdata: the data used for the new object. 
    :type obdata: 'bpy.data'
    :param operator: The operator, checked for location and rotation properties. 
    :type operator: 'bpy.types.Operator'
    :param name: Optional name 
    :type name: str
    :return:  the newly created object in the scene. 
    '''

    pass


def object_image_guess(obj, bm=None):
    '''Return a single image used by the object, first checking the texture-faces, then the material. 

    '''

    pass


def world_to_camera_view(scene: 'bpy.types.Scene', obj: 'bpy.types.Object',
                         coord: 'mathutils.Vector') -> 'mathutils.Vector':
    '''Takes shift-x/y, lens angle and sensor size into account as well as perspective/ortho projections. 

    :param scene: Scene to use for frame size. 
    :type scene: 'bpy.types.Scene'
    :param obj: Camera object. 
    :type obj: 'bpy.types.Object'
    :param coord: World space location. 
    :type coord: 'mathutils.Vector'
    :return:  a vector where X and Y map to the view plane and Z is the depth on the view axis. 
    '''

    pass



import sys
import typing


def flip(axis: typing.Union[int, str] = 'U'):
    '''Mirror all control points without inverting the lattice deform 

    :param axis: Flip Axis, Coordinates along this axis get flipped 
    :type axis: typing.Union[int, str]
    '''

    pass


def make_regular():
    '''Set UVW control points a uniform distance apart 

    '''

    pass


def select_all(action: typing.Union[int, str] = 'TOGGLE'):
    '''Change selection of all UVW control points 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_less():
    '''Deselect vertices at the boundary of each selection region 

    '''

    pass


def select_mirror(axis: typing.Set[typing.Union[int, str]] = {'X'},
                  extend: bool = False):
    '''Select mirrored lattice points 

    :param axis: Axis 
    :type axis: typing.Set[typing.Union[int, str]]
    :param extend: Extend, Extend the selection 
    :type extend: bool
    '''

    pass


def select_more():
    '''Select vertex directly linked to already selected ones 

    '''

    pass


def select_random(percent: float = 50.0,
                  seed: int = 0,
                  action: typing.Union[int, str] = 'SELECT'):
    '''Randomly select UVW control points 

    :param percent: Percent, Percentage of objects to select randomly 
    :type percent: float
    :param seed: Random Seed, Seed for the random number generator 
    :type seed: int
    :param action: Action, Selection action to executeSELECT Select, Select all elements.DESELECT Deselect, Deselect all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_ungrouped(extend: bool = False):
    '''Select vertices without a group 

    :param extend: Extend, Extend the selection 
    :type extend: bool
    '''

    pass



import sys
import typing


def average_normals(average_type: typing.Union[int, str] = 'CUSTOM_NORMAL',
                    weight: int = 50,
                    threshold: float = 0.01):
    '''Average custom normals of selected vertices 

    :param average_type: Type, Averaging methodCUSTOM_NORMAL Custom Normal, Take Average of vert Normals.FACE_AREA Face Area, Set all vert normals by Face Area.CORNER_ANGLE Corner Angle, Set all vert normals by Corner Angle. 
    :type average_type: typing.Union[int, str]
    :param weight: Weight, Weight applied per face 
    :type weight: int
    :param threshold: Threshold, Threshold value for different weights to be considered equal 
    :type threshold: float
    '''

    pass


def beautify_fill(angle_limit: float = 3.14159):
    '''Rearrange some faces to try to get less degenerated geometry 

    :param angle_limit: Max Angle, Angle limit 
    :type angle_limit: float
    '''

    pass


def bevel(offset_type: typing.Union[int, str] = 'OFFSET',
          offset: float = 0.0,
          offset_pct: float = 0.0,
          segments: int = 1,
          profile: float = 0.5,
          vertex_only: bool = False,
          clamp_overlap: bool = False,
          loop_slide: bool = True,
          mark_seam: bool = False,
          mark_sharp: bool = False,
          material: int = -1,
          harden_normals: bool = False,
          face_strength_mode: typing.Union[int, str] = 'NONE',
          miter_outer: typing.Union[int, str] = 'SHARP',
          miter_inner: typing.Union[int, str] = 'SHARP',
          spread: float = 0.1,
          release_confirm: bool = False):
    '''Cut into selected items at an angle to create flat or rounded bevel or chamfer 

    :param offset_type: Width Type, What distance Width measuresOFFSET Offset, Amount is offset of new edges from original.WIDTH Width, Amount is width of new face.DEPTH Depth, Amount is perpendicular distance from original edge to bevel face.PERCENT Percent, Amount is percent of adjacent edge length. 
    :type offset_type: typing.Union[int, str]
    :param offset: Width, Bevel amount 
    :type offset: float
    :param offset_pct: Width Percent, Bevel amount for percentage method 
    :type offset_pct: float
    :param segments: Segments, Segments for curved edge 
    :type segments: int
    :param profile: Profile, Controls profile shape (0.5 = round) 
    :type profile: float
    :param vertex_only: Vertex Only, Bevel only vertices 
    :type vertex_only: bool
    :param clamp_overlap: Clamp Overlap, Do not allow beveled edges/vertices to overlap each other 
    :type clamp_overlap: bool
    :param loop_slide: Loop Slide, Prefer slide along edge to even widths 
    :type loop_slide: bool
    :param mark_seam: Mark Seams, Mark Seams along beveled edges 
    :type mark_seam: bool
    :param mark_sharp: Mark Sharp, Mark beveled edges as sharp 
    :type mark_sharp: bool
    :param material: Material, Material for bevel faces (-1 means use adjacent faces) 
    :type material: int
    :param harden_normals: Harden Normals, Match normals of new faces to adjacent faces 
    :type harden_normals: bool
    :param face_strength_mode: Face Strength Mode, Whether to set face strength, and which faces to set face strength onNONE None, Do not set face strength.NEW New, Set face strength on new faces only.AFFECTED Affected, Set face strength on new and modified faces only.ALL All, Set face strength on all faces. 
    :type face_strength_mode: typing.Union[int, str]
    :param miter_outer: Outer Miter, Pattern to use for outside of mitersSHARP Sharp, Outside of miter is sharp.PATCH Patch, Outside of miter is squared-off patch.ARC Arc, Outside of miter is arc. 
    :type miter_outer: typing.Union[int, str]
    :param miter_inner: Inner Miter, Pattern to use for inside of mitersSHARP Sharp, Inside of miter is sharp.ARC Arc, Inside of miter is arc. 
    :type miter_inner: typing.Union[int, str]
    :param spread: Spread, Amount to spread arcs for arc inner miters 
    :type spread: float
    :param release_confirm: Confirm on Release 
    :type release_confirm: bool
    '''

    pass


def bisect(plane_co: float = (0.0, 0.0, 0.0),
           plane_no: float = (0.0, 0.0, 0.0),
           use_fill: bool = False,
           clear_inner: bool = False,
           clear_outer: bool = False,
           threshold: float = 0.0001,
           xstart: int = 0,
           xend: int = 0,
           ystart: int = 0,
           yend: int = 0,
           cursor: int = 1002):
    '''Cut geometry along a plane (click-drag to define plane) 

    :param plane_co: Plane Point, A point on the plane 
    :type plane_co: float
    :param plane_no: Plane Normal, The direction the plane points 
    :type plane_no: float
    :param use_fill: Fill, Fill in the cut 
    :type use_fill: bool
    :param clear_inner: Clear Inner, Remove geometry behind the plane 
    :type clear_inner: bool
    :param clear_outer: Clear Outer, Remove geometry in front of the plane 
    :type clear_outer: bool
    :param threshold: Axis Threshold, Preserves the existing geometry along the cut plane 
    :type threshold: float
    :param xstart: X Start 
    :type xstart: int
    :param xend: X End 
    :type xend: int
    :param ystart: Y Start 
    :type ystart: int
    :param yend: Y End 
    :type yend: int
    :param cursor: Cursor, Mouse cursor style to use during the modal operator 
    :type cursor: int
    '''

    pass


def blend_from_shape(shape: typing.Union[int, str] = '',
                     blend: float = 1.0,
                     add: bool = True):
    '''Blend in shape from a shape key 

    :param shape: Shape, Shape key to use for blending 
    :type shape: typing.Union[int, str]
    :param blend: Blend, Blending factor 
    :type blend: float
    :param add: Add, Add rather than blend between shapes 
    :type add: bool
    '''

    pass


def bridge_edge_loops(type: typing.Union[int, str] = 'SINGLE',
                      use_merge: bool = False,
                      merge_factor: float = 0.5,
                      twist_offset: int = 0,
                      number_cuts: int = 0,
                      interpolation: typing.Union[int, str] = 'PATH',
                      smoothness: float = 1.0,
                      profile_shape_factor: float = 0.0,
                      profile_shape: typing.Union[int, str] = 'SMOOTH'):
    '''Create a bridge of faces between two or more selected edge loops 

    :param type: Connect Loops, Method of bridging multiple loops 
    :type type: typing.Union[int, str]
    :param use_merge: Merge, Merge rather than creating faces 
    :type use_merge: bool
    :param merge_factor: Merge Factor 
    :type merge_factor: float
    :param twist_offset: Twist, Twist offset for closed loops 
    :type twist_offset: int
    :param number_cuts: Number of Cuts 
    :type number_cuts: int
    :param interpolation: Interpolation, Interpolation method 
    :type interpolation: typing.Union[int, str]
    :param smoothness: Smoothness, Smoothness factor 
    :type smoothness: float
    :param profile_shape_factor: Profile Factor, How much intermediary new edges are shrunk/expanded 
    :type profile_shape_factor: float
    :param profile_shape: Profile Shape, Shape of the profileSMOOTH Smooth, Smooth falloff.SPHERE Sphere, Spherical falloff.ROOT Root, Root falloff.INVERSE_SQUARE Inverse Square, Inverse Square falloff.SHARP Sharp, Sharp falloff.LINEAR Linear, Linear falloff. 
    :type profile_shape: typing.Union[int, str]
    '''

    pass


def colors_reverse():
    '''Flip direction of vertex colors inside faces 

    '''

    pass


def colors_rotate(use_ccw: bool = False):
    '''Rotate vertex colors inside faces 

    :param use_ccw: Counter Clockwise 
    :type use_ccw: bool
    '''

    pass


def convex_hull(delete_unused: bool = True,
                use_existing_faces: bool = True,
                make_holes: bool = False,
                join_triangles: bool = True,
                face_threshold: float = 0.698132,
                shape_threshold: float = 0.698132,
                uvs: bool = False,
                vcols: bool = False,
                seam: bool = False,
                sharp: bool = False,
                materials: bool = False):
    '''Enclose selected vertices in a convex polyhedron 

    :param delete_unused: Delete Unused, Delete selected elements that are not used by the hull 
    :type delete_unused: bool
    :param use_existing_faces: Use Existing Faces, Skip hull triangles that are covered by a pre-existing face 
    :type use_existing_faces: bool
    :param make_holes: Make Holes, Delete selected faces that are used by the hull 
    :type make_holes: bool
    :param join_triangles: Join Triangles, Merge adjacent triangles into quads 
    :type join_triangles: bool
    :param face_threshold: Max Face Angle, Face angle limit 
    :type face_threshold: float
    :param shape_threshold: Max Shape Angle, Shape angle limit 
    :type shape_threshold: float
    :param uvs: Compare UVs 
    :type uvs: bool
    :param vcols: Compare VCols 
    :type vcols: bool
    :param seam: Compare Seam 
    :type seam: bool
    :param sharp: Compare Sharp 
    :type sharp: bool
    :param materials: Compare Materials 
    :type materials: bool
    '''

    pass


def customdata_custom_splitnormals_add():
    '''Add a custom split normals layer, if none exists yet 

    '''

    pass


def customdata_custom_splitnormals_clear():
    '''Remove the custom split normals layer, if it exists 

    '''

    pass


def customdata_mask_clear():
    '''Clear vertex sculpt masking data from the mesh 

    '''

    pass


def customdata_skin_add():
    '''Add a vertex skin layer 

    '''

    pass


def customdata_skin_clear():
    '''Clear vertex skin layer 

    '''

    pass


def decimate(ratio: float = 1.0,
             use_vertex_group: bool = False,
             vertex_group_factor: float = 1.0,
             invert_vertex_group: bool = False,
             use_symmetry: bool = False,
             symmetry_axis: typing.Union[int, str] = 'Y'):
    '''Simplify geometry by collapsing edges 

    :param ratio: Ratio 
    :type ratio: float
    :param use_vertex_group: Vertex Group, Use active vertex group as an influence 
    :type use_vertex_group: bool
    :param vertex_group_factor: Weight, Vertex group strength 
    :type vertex_group_factor: float
    :param invert_vertex_group: Invert, Invert vertex group influence 
    :type invert_vertex_group: bool
    :param use_symmetry: Symmetry, Maintain symmetry on an axis 
    :type use_symmetry: bool
    :param symmetry_axis: Axis, Axis of symmetry 
    :type symmetry_axis: typing.Union[int, str]
    '''

    pass


def delete(type: typing.Union[int, str] = 'VERT'):
    '''Delete selected vertices, edges or faces 

    :param type: Type, Method used for deleting mesh data 
    :type type: typing.Union[int, str]
    '''

    pass


def delete_edgeloop(use_face_split: bool = True):
    '''Delete an edge loop by merging the faces on each side 

    :param use_face_split: Face Split, Split off face corners to maintain surrounding geometry 
    :type use_face_split: bool
    '''

    pass


def delete_loose(use_verts: bool = True,
                 use_edges: bool = True,
                 use_faces: bool = False):
    '''Delete loose vertices, edges or faces 

    :param use_verts: Vertices, Remove loose vertices 
    :type use_verts: bool
    :param use_edges: Edges, Remove loose edges 
    :type use_edges: bool
    :param use_faces: Faces, Remove loose faces 
    :type use_faces: bool
    '''

    pass


def dissolve_degenerate(threshold: float = 0.0001):
    '''Dissolve zero area faces and zero length edges 

    :param threshold: Merge Distance, Minimum distance between elements to merge 
    :type threshold: float
    '''

    pass


def dissolve_edges(use_verts: bool = True, use_face_split: bool = False):
    '''Dissolve edges, merging faces 

    :param use_verts: Dissolve Verts, Dissolve remaining vertices 
    :type use_verts: bool
    :param use_face_split: Face Split, Split off face corners to maintain surrounding geometry 
    :type use_face_split: bool
    '''

    pass


def dissolve_faces(use_verts: bool = False):
    '''Dissolve faces 

    :param use_verts: Dissolve Verts, Dissolve remaining vertices 
    :type use_verts: bool
    '''

    pass


def dissolve_limited(angle_limit: float = 0.0872665,
                     use_dissolve_boundaries: bool = False,
                     delimit: typing.Set[typing.Union[int, str]] = {'NORMAL'}):
    '''Dissolve selected edges and verts, limited by the angle of surrounding geometry 

    :param angle_limit: Max Angle, Angle limit 
    :type angle_limit: float
    :param use_dissolve_boundaries: All Boundaries, Dissolve all vertices inbetween face boundaries 
    :type use_dissolve_boundaries: bool
    :param delimit: Delimit, Delimit dissolve operationNORMAL Regular, Delimit by face directions.MATERIAL Material, Delimit by face material.SEAM Seam, Delimit by edge seams.SHARP Sharp, Delimit by sharp edges.UV UVs, Delimit by UV coordinates. 
    :type delimit: typing.Set[typing.Union[int, str]]
    '''

    pass


def dissolve_mode(use_verts: bool = False,
                  use_face_split: bool = False,
                  use_boundary_tear: bool = False):
    '''Dissolve geometry based on the selection mode 

    :param use_verts: Dissolve Verts, Dissolve remaining vertices 
    :type use_verts: bool
    :param use_face_split: Face Split, Split off face corners to maintain surrounding geometry 
    :type use_face_split: bool
    :param use_boundary_tear: Tear Boundary, Split off face corners instead of merging faces 
    :type use_boundary_tear: bool
    '''

    pass


def dissolve_verts(use_face_split: bool = False,
                   use_boundary_tear: bool = False):
    '''Dissolve verts, merge edges and faces 

    :param use_face_split: Face Split, Split off face corners to maintain surrounding geometry 
    :type use_face_split: bool
    :param use_boundary_tear: Tear Boundary, Split off face corners instead of merging faces 
    :type use_boundary_tear: bool
    '''

    pass


def dupli_extrude_cursor(rotate_source: bool = True):
    '''Duplicate and extrude selected vertices, edges or faces towards the mouse cursor 

    :param rotate_source: Rotate Source, Rotate initial selection giving better shape 
    :type rotate_source: bool
    '''

    pass


def duplicate(mode: int = 1):
    '''Duplicate selected vertices, edges or faces 

    :param mode: Mode 
    :type mode: int
    '''

    pass


def duplicate_move(MESH_OT_duplicate=None, TRANSFORM_OT_translate=None):
    '''Duplicate mesh and move 

    :param MESH_OT_duplicate: Duplicate, Duplicate selected vertices, edges or faces 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def edge_collapse():
    '''Collapse selected edges 

    '''

    pass


def edge_face_add():
    '''Add an edge or face to selected 

    '''

    pass


def edge_rotate(use_ccw: bool = False):
    '''Rotate selected edge or adjoining faces 

    :param use_ccw: Counter Clockwise 
    :type use_ccw: bool
    '''

    pass


def edge_split():
    '''Split selected edges so that each neighbor face gets its own copy 

    '''

    pass


def edgering_select(extend: bool = False,
                    deselect: bool = False,
                    toggle: bool = False,
                    ring: bool = True):
    '''Select an edge ring 

    :param extend: Extend, Extend the selection 
    :type extend: bool
    :param deselect: Deselect, Remove from the selection 
    :type deselect: bool
    :param toggle: Toggle Select, Toggle the selection 
    :type toggle: bool
    :param ring: Select Ring, Select ring 
    :type ring: bool
    '''

    pass


def edges_select_sharp(sharpness: float = 0.523599):
    '''Select all sharp-enough edges 

    :param sharpness: Sharpness 
    :type sharpness: float
    '''

    pass


def extrude_context(use_normal_flip: bool = False, mirror: bool = False):
    '''Extrude selection 

    :param use_normal_flip: Flip Normals 
    :type use_normal_flip: bool
    :param mirror: Mirror Editing 
    :type mirror: bool
    '''

    pass


def extrude_context_move(MESH_OT_extrude_context=None,
                         TRANSFORM_OT_translate=None):
    '''Extrude region together along the average normal 

    :param MESH_OT_extrude_context: Extrude Context, Extrude selection 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def extrude_edges_indiv(use_normal_flip: bool = False, mirror: bool = False):
    '''Extrude individual edges only 

    :param use_normal_flip: Flip Normals 
    :type use_normal_flip: bool
    :param mirror: Mirror Editing 
    :type mirror: bool
    '''

    pass


def extrude_edges_move(MESH_OT_extrude_edges_indiv=None,
                       TRANSFORM_OT_translate=None):
    '''Extrude edges and move result 

    :param MESH_OT_extrude_edges_indiv: Extrude Only Edges, Extrude individual edges only 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def extrude_faces_indiv(mirror: bool = False):
    '''Extrude individual faces only 

    :param mirror: Mirror Editing 
    :type mirror: bool
    '''

    pass


def extrude_faces_move(MESH_OT_extrude_faces_indiv=None,
                       TRANSFORM_OT_shrink_fatten=None):
    '''Extrude each individual face separately along local normals 

    :param MESH_OT_extrude_faces_indiv: Extrude Individual Faces, Extrude individual faces only 
    :param TRANSFORM_OT_shrink_fatten: Shrink/Fatten, Shrink/fatten selected vertices along normals 
    '''

    pass


def extrude_region(use_normal_flip: bool = False, mirror: bool = False):
    '''Extrude region of faces 

    :param use_normal_flip: Flip Normals 
    :type use_normal_flip: bool
    :param mirror: Mirror Editing 
    :type mirror: bool
    '''

    pass


def extrude_region_move(MESH_OT_extrude_region=None,
                        TRANSFORM_OT_translate=None):
    '''Extrude region and move result 

    :param MESH_OT_extrude_region: Extrude Region, Extrude region of faces 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def extrude_region_shrink_fatten(MESH_OT_extrude_region=None,
                                 TRANSFORM_OT_shrink_fatten=None):
    '''Extrude region together along local normals 

    :param MESH_OT_extrude_region: Extrude Region, Extrude region of faces 
    :param TRANSFORM_OT_shrink_fatten: Shrink/Fatten, Shrink/fatten selected vertices along normals 
    '''

    pass


def extrude_repeat(offset: float = 2.0, steps: int = 10):
    '''Extrude selected vertices, edges or faces repeatedly 

    :param offset: Offset 
    :type offset: float
    :param steps: Steps 
    :type steps: int
    '''

    pass


def extrude_vertices_move(MESH_OT_extrude_verts_indiv=None,
                          TRANSFORM_OT_translate=None):
    '''Extrude vertices and move result 

    :param MESH_OT_extrude_verts_indiv: Extrude Only Vertices, Extrude individual vertices only 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def extrude_verts_indiv(mirror: bool = False):
    '''Extrude individual vertices only 

    :param mirror: Mirror Editing 
    :type mirror: bool
    '''

    pass


def face_make_planar(factor: float = 1.0, repeat: int = 1):
    '''Flatten selected faces 

    :param factor: Factor 
    :type factor: float
    :param repeat: Iterations 
    :type repeat: int
    '''

    pass


def face_split_by_edges():
    '''Weld loose edges into faces (splitting them into new faces) 

    '''

    pass


def faces_mirror_uv(direction: typing.Union[int, str] = 'POSITIVE',
                    precision: int = 3):
    '''Copy mirror UV coordinates on the X axis based on a mirrored mesh 

    :param direction: Axis Direction 
    :type direction: typing.Union[int, str]
    :param precision: Precision, Tolerance for finding vertex duplicates 
    :type precision: int
    '''

    pass


def faces_select_linked_flat(sharpness: float = 0.0174533):
    '''Select linked faces by angle 

    :param sharpness: Sharpness 
    :type sharpness: float
    '''

    pass


def faces_shade_flat():
    '''Display faces flat 

    '''

    pass


def faces_shade_smooth():
    '''Display faces smooth (using vertex normals) 

    '''

    pass


def fill(use_beauty: bool = True):
    '''Fill a selected edge loop with faces 

    :param use_beauty: Beauty, Use best triangulation division 
    :type use_beauty: bool
    '''

    pass


def fill_grid(span: int = 1, offset: int = 0, use_interp_simple: bool = False):
    '''Fill grid from two loops 

    :param span: Span, Number of grid columns 
    :type span: int
    :param offset: Offset, Vertex that is the corner of the grid 
    :type offset: int
    :param use_interp_simple: Simple Blending, Use simple interpolation of grid vertices 
    :type use_interp_simple: bool
    '''

    pass


def fill_holes(sides: int = 4):
    '''Fill in holes (boundary edge loops) 

    :param sides: Sides, Number of sides in hole required to fill (zero fills all holes) 
    :type sides: int
    '''

    pass


def flip_normals():
    '''Flip the direction of selected faces’ normals (and of their vertices) 

    '''

    pass


def hide(unselected: bool = False):
    '''Hide (un)selected vertices, edges or faces 

    :param unselected: Unselected, Hide unselected rather than selected 
    :type unselected: bool
    '''

    pass


def inset(use_boundary: bool = True,
          use_even_offset: bool = True,
          use_relative_offset: bool = False,
          use_edge_rail: bool = False,
          thickness: float = 0.0,
          depth: float = 0.0,
          use_outset: bool = False,
          use_select_inset: bool = False,
          use_individual: bool = False,
          use_interpolate: bool = True,
          release_confirm: bool = False):
    '''Inset new faces into selected faces 

    :param use_boundary: Boundary, Inset face boundaries 
    :type use_boundary: bool
    :param use_even_offset: Offset Even, Scale the offset to give more even thickness 
    :type use_even_offset: bool
    :param use_relative_offset: Offset Relative, Scale the offset by surrounding geometry 
    :type use_relative_offset: bool
    :param use_edge_rail: Edge Rail, Inset the region along existing edges 
    :type use_edge_rail: bool
    :param thickness: Thickness 
    :type thickness: float
    :param depth: Depth 
    :type depth: float
    :param use_outset: Outset, Outset rather than inset 
    :type use_outset: bool
    :param use_select_inset: Select Outer, Select the new inset faces 
    :type use_select_inset: bool
    :param use_individual: Individual, Individual Face Inset 
    :type use_individual: bool
    :param use_interpolate: Interpolate, Blend face data across the inset 
    :type use_interpolate: bool
    :param release_confirm: Confirm on Release 
    :type release_confirm: bool
    '''

    pass


def intersect(mode: typing.Union[int, str] = 'SELECT_UNSELECT',
              separate_mode: typing.Union[int, str] = 'CUT',
              threshold: float = 1e-06):
    '''Cut an intersection into faces 

    :param mode: SourceSELECT Self Intersect, Self intersect selected faces.SELECT_UNSELECT Selected/Unselected, Intersect selected with unselected faces. 
    :type mode: typing.Union[int, str]
    :param separate_mode: Separate ModeALL All, Separate all geometry from intersections.CUT Cut, Cut into geometry keeping each side separate (Selected/Unselected only).NONE Merge, Merge all geometry from the intersection. 
    :type separate_mode: typing.Union[int, str]
    :param threshold: Merge threshold 
    :type threshold: float
    '''

    pass


def intersect_boolean(operation: typing.Union[int, str] = 'DIFFERENCE',
                      use_swap: bool = False,
                      threshold: float = 1e-06):
    '''Cut solid geometry from selected to unselected 

    :param operation: Boolean 
    :type operation: typing.Union[int, str]
    :param use_swap: Swap, Use with difference intersection to swap which side is kept 
    :type use_swap: bool
    :param threshold: Merge threshold 
    :type threshold: float
    '''

    pass


def knife_project(cut_through: bool = False):
    '''Use other objects outlines & boundaries to project knife cuts 

    :param cut_through: Cut through, Cut through all faces, not just visible ones 
    :type cut_through: bool
    '''

    pass


def knife_tool(use_occlude_geometry: bool = True,
               only_selected: bool = False,
               wait_for_input: bool = True):
    '''Cut new topology 

    :param use_occlude_geometry: Occlude Geometry, Only cut the front most geometry 
    :type use_occlude_geometry: bool
    :param only_selected: Only Selected, Only cut selected geometry 
    :type only_selected: bool
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    '''

    pass


def loop_multi_select(ring: bool = False):
    '''Select a loop of connected edges by connection type 

    :param ring: Ring 
    :type ring: bool
    '''

    pass


def loop_select(extend: bool = False,
                deselect: bool = False,
                toggle: bool = False,
                ring: bool = False):
    '''Select a loop of connected edges 

    :param extend: Extend Select, Extend the selection 
    :type extend: bool
    :param deselect: Deselect, Remove from the selection 
    :type deselect: bool
    :param toggle: Toggle Select, Toggle the selection 
    :type toggle: bool
    :param ring: Select Ring, Select ring 
    :type ring: bool
    '''

    pass


def loop_to_region(select_bigger: bool = False):
    '''Select region of faces inside of a selected loop of edges 

    :param select_bigger: Select Bigger, Select bigger regions instead of smaller ones 
    :type select_bigger: bool
    '''

    pass


def loopcut(number_cuts: int = 1,
            smoothness: float = 0.0,
            falloff: typing.Union[int, str] = 'INVERSE_SQUARE',
            object_index: int = -1,
            edge_index: int = -1,
            mesh_select_mode_init=(False, False, False)):
    '''Add a new loop between existing loops 

    :param number_cuts: Number of Cuts 
    :type number_cuts: int
    :param smoothness: Smoothness, Smoothness factor 
    :type smoothness: float
    :param falloff: Falloff, Falloff type the featherSMOOTH Smooth, Smooth falloff.SPHERE Sphere, Spherical falloff.ROOT Root, Root falloff.INVERSE_SQUARE Inverse Square, Inverse Square falloff.SHARP Sharp, Sharp falloff.LINEAR Linear, Linear falloff. 
    :type falloff: typing.Union[int, str]
    :param object_index: Object Index 
    :type object_index: int
    :param edge_index: Edge Index 
    :type edge_index: int
    '''

    pass


def loopcut_slide(MESH_OT_loopcut=None, TRANSFORM_OT_edge_slide=None):
    '''Cut mesh loop and slide it 

    :param MESH_OT_loopcut: Loop Cut, Add a new loop between existing loops 
    :param TRANSFORM_OT_edge_slide: Edge Slide, Slide an edge loop along a mesh 
    '''

    pass


def mark_freestyle_edge(clear: bool = False):
    '''(Un)mark selected edges as Freestyle feature edges 

    :param clear: Clear 
    :type clear: bool
    '''

    pass


def mark_freestyle_face(clear: bool = False):
    '''(Un)mark selected faces for exclusion from Freestyle feature edge detection 

    :param clear: Clear 
    :type clear: bool
    '''

    pass


def mark_seam(clear: bool = False):
    '''(Un)mark selected edges as a seam 

    :param clear: Clear 
    :type clear: bool
    '''

    pass


def mark_sharp(clear: bool = False, use_verts: bool = False):
    '''(Un)mark selected edges as sharp 

    :param clear: Clear 
    :type clear: bool
    :param use_verts: Vertices, Consider vertices instead of edges to select which edges to (un)tag as sharp 
    :type use_verts: bool
    '''

    pass


def merge(type: typing.Union[int, str] = 'CENTER', uvs: bool = False):
    '''Merge selected vertices 

    :param type: Type, Merge method to use 
    :type type: typing.Union[int, str]
    :param uvs: UVs, Move UVs according to merge 
    :type uvs: bool
    '''

    pass


def merge_normals():
    '''Merge custom normals of selected vertices 

    '''

    pass


def mod_weighted_strength(set: bool = False,
                          face_strength: typing.Union[int, str] = 'MEDIUM'):
    '''Set/Get strength of face (used in Weighted Normal modifier) 

    :param set: Set value, Set Value of faces 
    :type set: bool
    :param face_strength: Face Strength, Strength to use for assigning or selecting face influence for weighted normal modifier 
    :type face_strength: typing.Union[int, str]
    '''

    pass


def normals_make_consistent(inside: bool = False):
    '''Make face and vertex normals point either outside or inside the mesh 

    :param inside: Inside 
    :type inside: bool
    '''

    pass


def normals_tools(mode: typing.Union[int, str] = 'COPY',
                  absolute: bool = False):
    '''Custom normals tools using Normal Vector of UI 

    :param mode: Mode, Mode of tools taking input from InterfaceCOPY Copy Normal, Copy normal to buffer.PASTE Paste Normal, Paste normal from buffer.ADD Add Normal, Add normal vector with selection.MULTIPLY Multiply Normal, Multiply normal vector with selection.RESET Reset Normal, Reset buffer and/or normal of selected element. 
    :type mode: typing.Union[int, str]
    :param absolute: Absolute Coordinates, Copy Absolute coordinates or Normal vector 
    :type absolute: bool
    '''

    pass


def offset_edge_loops(use_cap_endpoint: bool = False):
    '''Create offset edge loop from the current selection 

    :param use_cap_endpoint: Cap Endpoint, Extend loop around end-points 
    :type use_cap_endpoint: bool
    '''

    pass


def offset_edge_loops_slide(MESH_OT_offset_edge_loops=None,
                            TRANSFORM_OT_edge_slide=None):
    '''Offset edge loop slide 

    :param MESH_OT_offset_edge_loops: Offset Edge Loop, Create offset edge loop from the current selection 
    :param TRANSFORM_OT_edge_slide: Edge Slide, Slide an edge loop along a mesh 
    '''

    pass


def point_normals(mode: typing.Union[int, str] = 'COORDINATES',
                  invert: bool = False,
                  align: bool = False,
                  target_location: float = (0.0, 0.0, 0.0),
                  spherize: bool = False,
                  spherize_strength: float = 0.1):
    '''Point selected custom normals to specified Target 

    :param mode: Mode, How to define coordinates to point custom normals toCOORDINATES Coordinates, Use static coordinates (defined by various means).MOUSE Mouse, Follow mouse cursor. 
    :type mode: typing.Union[int, str]
    :param invert: Invert, Invert affected normals 
    :type invert: bool
    :param align: Align, Make all affected normals parallel 
    :type align: bool
    :param target_location: Target, Target location to which normals will point 
    :type target_location: float
    :param spherize: Spherize, Interpolate between original and new normals 
    :type spherize: bool
    :param spherize_strength: Spherize Strength, Ratio of spherized normal to original normal 
    :type spherize_strength: float
    '''

    pass


def poke(offset: float = 0.0,
         use_relative_offset: bool = False,
         center_mode: typing.Union[int, str] = 'MEDIAN_WEIGHTED'):
    '''Split a face into a fan 

    :param offset: Poke Offset, Poke Offset 
    :type offset: float
    :param use_relative_offset: Offset Relative, Scale the offset by surrounding geometry 
    :type use_relative_offset: bool
    :param center_mode: Poke Center, Poke Face Center CalculationMEDIAN_WEIGHTED Weighted Median, Weighted median face center.MEDIAN Median, Median face center.BOUNDS Bounds, Face bounds center. 
    :type center_mode: typing.Union[int, str]
    '''

    pass


def polybuild_dissolve_at_cursor():
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    '''

    pass


def polybuild_face_at_cursor(
        mirror: bool = False,
        use_proportional_edit: bool = False,
        proportional_edit_falloff: typing.Union[int, str] = 'SMOOTH',
        proportional_size: float = 1.0,
        use_proportional_connected: bool = False,
        use_proportional_projected: bool = False,
        release_confirm: bool = False,
        use_accurate: bool = False):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param mirror: Mirror Editing 
    :type mirror: bool
    :param use_proportional_edit: Proportional Editing 
    :type use_proportional_edit: bool
    :param proportional_edit_falloff: Proportional Falloff, Falloff type for proportional editing modeSMOOTH Smooth, Smooth falloff.SPHERE Sphere, Spherical falloff.ROOT Root, Root falloff.INVERSE_SQUARE Inverse Square, Inverse Square falloff.SHARP Sharp, Sharp falloff.LINEAR Linear, Linear falloff.CONSTANT Constant, Constant falloff.RANDOM Random, Random falloff. 
    :type proportional_edit_falloff: typing.Union[int, str]
    :param proportional_size: Proportional Size 
    :type proportional_size: float
    :param use_proportional_connected: Connected 
    :type use_proportional_connected: bool
    :param use_proportional_projected: Projected (2D) 
    :type use_proportional_projected: bool
    :param release_confirm: Confirm on Release, Always confirm operation when releasing button 
    :type release_confirm: bool
    :param use_accurate: Accurate, Use accurate transformation 
    :type use_accurate: bool
    '''

    pass


def polybuild_face_at_cursor_move(MESH_OT_polybuild_face_at_cursor=None,
                                  TRANSFORM_OT_translate=None):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param MESH_OT_polybuild_face_at_cursor: Poly Build Face at Cursor 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def polybuild_split_at_cursor(
        mirror: bool = False,
        use_proportional_edit: bool = False,
        proportional_edit_falloff: typing.Union[int, str] = 'SMOOTH',
        proportional_size: float = 1.0,
        use_proportional_connected: bool = False,
        use_proportional_projected: bool = False,
        release_confirm: bool = False,
        use_accurate: bool = False):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param mirror: Mirror Editing 
    :type mirror: bool
    :param use_proportional_edit: Proportional Editing 
    :type use_proportional_edit: bool
    :param proportional_edit_falloff: Proportional Falloff, Falloff type for proportional editing modeSMOOTH Smooth, Smooth falloff.SPHERE Sphere, Spherical falloff.ROOT Root, Root falloff.INVERSE_SQUARE Inverse Square, Inverse Square falloff.SHARP Sharp, Sharp falloff.LINEAR Linear, Linear falloff.CONSTANT Constant, Constant falloff.RANDOM Random, Random falloff. 
    :type proportional_edit_falloff: typing.Union[int, str]
    :param proportional_size: Proportional Size 
    :type proportional_size: float
    :param use_proportional_connected: Connected 
    :type use_proportional_connected: bool
    :param use_proportional_projected: Projected (2D) 
    :type use_proportional_projected: bool
    :param release_confirm: Confirm on Release, Always confirm operation when releasing button 
    :type release_confirm: bool
    :param use_accurate: Accurate, Use accurate transformation 
    :type use_accurate: bool
    '''

    pass


def polybuild_split_at_cursor_move(MESH_OT_polybuild_split_at_cursor=None,
                                   TRANSFORM_OT_translate=None):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param MESH_OT_polybuild_split_at_cursor: Poly Build Split at Cursor 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def primitive_circle_add(vertices: int = 32,
                         radius: float = 1.0,
                         fill_type: typing.Union[int, str] = 'NOTHING',
                         calc_uvs: bool = True,
                         enter_editmode: bool = False,
                         align: typing.Union[int, str] = 'WORLD',
                         location: float = (0.0, 0.0, 0.0),
                         rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a circle mesh 

    :param vertices: Vertices 
    :type vertices: int
    :param radius: Radius 
    :type radius: float
    :param fill_type: Fill TypeNOTHING Nothing, Don’t fill at all.NGON Ngon, Use ngons.TRIFAN Triangle Fan, Use triangle fans. 
    :type fill_type: typing.Union[int, str]
    :param calc_uvs: Generate UVs, Generate a default UV map 
    :type calc_uvs: bool
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def primitive_cone_add(vertices: int = 32,
                       radius1: float = 1.0,
                       radius2: float = 0.0,
                       depth: float = 2.0,
                       end_fill_type: typing.Union[int, str] = 'NGON',
                       calc_uvs: bool = True,
                       enter_editmode: bool = False,
                       align: typing.Union[int, str] = 'WORLD',
                       location: float = (0.0, 0.0, 0.0),
                       rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a conic mesh 

    :param vertices: Vertices 
    :type vertices: int
    :param radius1: Radius 1 
    :type radius1: float
    :param radius2: Radius 2 
    :type radius2: float
    :param depth: Depth 
    :type depth: float
    :param end_fill_type: Base Fill TypeNOTHING Nothing, Don’t fill at all.NGON Ngon, Use ngons.TRIFAN Triangle Fan, Use triangle fans. 
    :type end_fill_type: typing.Union[int, str]
    :param calc_uvs: Generate UVs, Generate a default UV map 
    :type calc_uvs: bool
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def primitive_cube_add(size: float = 2.0,
                       calc_uvs: bool = True,
                       enter_editmode: bool = False,
                       align: typing.Union[int, str] = 'WORLD',
                       location: float = (0.0, 0.0, 0.0),
                       rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a cube mesh 

    :param size: Size 
    :type size: float
    :param calc_uvs: Generate UVs, Generate a default UV map 
    :type calc_uvs: bool
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def primitive_cube_add_gizmo(
        calc_uvs: bool = True,
        enter_editmode: bool = False,
        align: typing.Union[int, str] = 'WORLD',
        location: float = (0.0, 0.0, 0.0),
        rotation: float = (0.0, 0.0, 0.0),
        matrix: float = ((0.0, 0.0, 0.0, 0.0), (0.0, 0.0, 0.0, 0.0),
                         (0.0, 0.0, 0.0, 0.0), (0.0, 0.0, 0.0, 0.0))):
    '''Construct a cube mesh 

    :param calc_uvs: Generate UVs, Generate a default UV map 
    :type calc_uvs: bool
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    :param matrix: Matrix 
    :type matrix: float
    '''

    pass


def primitive_cylinder_add(vertices: int = 32,
                           radius: float = 1.0,
                           depth: float = 2.0,
                           end_fill_type: typing.Union[int, str] = 'NGON',
                           calc_uvs: bool = True,
                           enter_editmode: bool = False,
                           align: typing.Union[int, str] = 'WORLD',
                           location: float = (0.0, 0.0, 0.0),
                           rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a cylinder mesh 

    :param vertices: Vertices 
    :type vertices: int
    :param radius: Radius 
    :type radius: float
    :param depth: Depth 
    :type depth: float
    :param end_fill_type: Cap Fill TypeNOTHING Nothing, Don’t fill at all.NGON Ngon, Use ngons.TRIFAN Triangle Fan, Use triangle fans. 
    :type end_fill_type: typing.Union[int, str]
    :param calc_uvs: Generate UVs, Generate a default UV map 
    :type calc_uvs: bool
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def primitive_grid_add(x_subdivisions: int = 10,
                       y_subdivisions: int = 10,
                       size: float = 2.0,
                       calc_uvs: bool = True,
                       enter_editmode: bool = False,
                       align: typing.Union[int, str] = 'WORLD',
                       location: float = (0.0, 0.0, 0.0),
                       rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a grid mesh 

    :param x_subdivisions: X Subdivisions 
    :type x_subdivisions: int
    :param y_subdivisions: Y Subdivisions 
    :type y_subdivisions: int
    :param size: Size 
    :type size: float
    :param calc_uvs: Generate UVs, Generate a default UV map 
    :type calc_uvs: bool
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def primitive_ico_sphere_add(subdivisions: int = 2,
                             radius: float = 1.0,
                             calc_uvs: bool = True,
                             enter_editmode: bool = False,
                             align: typing.Union[int, str] = 'WORLD',
                             location: float = (0.0, 0.0, 0.0),
                             rotation: float = (0.0, 0.0, 0.0)):
    '''Construct an Icosphere mesh 

    :param subdivisions: Subdivisions 
    :type subdivisions: int
    :param radius: Radius 
    :type radius: float
    :param calc_uvs: Generate UVs, Generate a default UV map 
    :type calc_uvs: bool
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def primitive_monkey_add(size: float = 2.0,
                         calc_uvs: bool = True,
                         enter_editmode: bool = False,
                         align: typing.Union[int, str] = 'WORLD',
                         location: float = (0.0, 0.0, 0.0),
                         rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a Suzanne mesh 

    :param size: Size 
    :type size: float
    :param calc_uvs: Generate UVs, Generate a default UV map 
    :type calc_uvs: bool
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def primitive_plane_add(size: float = 2.0,
                        calc_uvs: bool = True,
                        enter_editmode: bool = False,
                        align: typing.Union[int, str] = 'WORLD',
                        location: float = (0.0, 0.0, 0.0),
                        rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a filled planar mesh with 4 vertices 

    :param size: Size 
    :type size: float
    :param calc_uvs: Generate UVs, Generate a default UV map 
    :type calc_uvs: bool
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def primitive_torus_add(align: typing.Union[int, str] = 'WORLD',
                        location: float = (0.0, 0.0, 0.0),
                        rotation: float = (0.0, 0.0, 0.0),
                        major_segments: int = 48,
                        minor_segments: int = 12,
                        mode: typing.Union[int, str] = 'MAJOR_MINOR',
                        major_radius: float = 1.0,
                        minor_radius: float = 0.25,
                        abso_major_rad: float = 1.25,
                        abso_minor_rad: float = 0.75,
                        generate_uvs: bool = True):
    '''Add a torus mesh 

    :param align: AlignWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location 
    :type location: float
    :param rotation: Rotation 
    :type rotation: float
    :param major_segments: Major Segments, Number of segments for the main ring of the torus 
    :type major_segments: int
    :param minor_segments: Minor Segments, Number of segments for the minor ring of the torus 
    :type minor_segments: int
    :param mode: Torus DimensionsMAJOR_MINOR Major/Minor, Use the major/minor radii for torus dimensions.EXT_INT Exterior/Interior, Use the exterior/interior radii for torus dimensions. 
    :type mode: typing.Union[int, str]
    :param major_radius: Major Radius, Radius from the origin to the center of the cross sections 
    :type major_radius: float
    :param minor_radius: Minor Radius, Radius of the torus’ cross section 
    :type minor_radius: float
    :param abso_major_rad: Exterior Radius, Total Exterior Radius of the torus 
    :type abso_major_rad: float
    :param abso_minor_rad: Interior Radius, Total Interior Radius of the torus 
    :type abso_minor_rad: float
    :param generate_uvs: Generate UVs, Generate a default UV map 
    :type generate_uvs: bool
    '''

    pass


def primitive_uv_sphere_add(segments: int = 32,
                            ring_count: int = 16,
                            radius: float = 1.0,
                            calc_uvs: bool = True,
                            enter_editmode: bool = False,
                            align: typing.Union[int, str] = 'WORLD',
                            location: float = (0.0, 0.0, 0.0),
                            rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a UV sphere mesh 

    :param segments: Segments 
    :type segments: int
    :param ring_count: Rings 
    :type ring_count: int
    :param radius: Radius 
    :type radius: float
    :param calc_uvs: Generate UVs, Generate a default UV map 
    :type calc_uvs: bool
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def quads_convert_to_tris(quad_method: typing.Union[int, str] = 'BEAUTY',
                          ngon_method: typing.Union[int, str] = 'BEAUTY'):
    '''Triangulate selected faces 

    :param quad_method: Quad Method, Method for splitting the quads into trianglesBEAUTY Beauty , Split the quads in nice triangles, slower method.FIXED Fixed, Split the quads on the first and third vertices.FIXED_ALTERNATE Fixed Alternate, Split the quads on the 2nd and 4th vertices.SHORTEST_DIAGONAL Shortest Diagonal, Split the quads based on the distance between the vertices. 
    :type quad_method: typing.Union[int, str]
    :param ngon_method: Polygon Method, Method for splitting the polygons into trianglesBEAUTY Beauty, Arrange the new triangles evenly (slow).CLIP Clip, Split the polygons with an ear clipping algorithm. 
    :type ngon_method: typing.Union[int, str]
    '''

    pass


def region_to_loop():
    '''Select boundary edges around the selected faces 

    '''

    pass


def remove_doubles(threshold: float = 0.0001, use_unselected: bool = False):
    '''Merge vertices based on their proximity 

    :param threshold: Merge Distance, Minimum distance between elements to merge 
    :type threshold: float
    :param use_unselected: Unselected, Merge selected to other unselected vertices 
    :type use_unselected: bool
    '''

    pass


def reveal(select: bool = True):
    '''Reveal all hidden vertices, edges and faces 

    :param select: Select 
    :type select: bool
    '''

    pass


def rip(mirror: bool = False,
        use_proportional_edit: bool = False,
        proportional_edit_falloff: typing.Union[int, str] = 'SMOOTH',
        proportional_size: float = 1.0,
        use_proportional_connected: bool = False,
        use_proportional_projected: bool = False,
        release_confirm: bool = False,
        use_accurate: bool = False,
        use_fill: bool = False):
    '''Disconnect vertex or edges from connected geometry 

    :param mirror: Mirror Editing 
    :type mirror: bool
    :param use_proportional_edit: Proportional Editing 
    :type use_proportional_edit: bool
    :param proportional_edit_falloff: Proportional Falloff, Falloff type for proportional editing modeSMOOTH Smooth, Smooth falloff.SPHERE Sphere, Spherical falloff.ROOT Root, Root falloff.INVERSE_SQUARE Inverse Square, Inverse Square falloff.SHARP Sharp, Sharp falloff.LINEAR Linear, Linear falloff.CONSTANT Constant, Constant falloff.RANDOM Random, Random falloff. 
    :type proportional_edit_falloff: typing.Union[int, str]
    :param proportional_size: Proportional Size 
    :type proportional_size: float
    :param use_proportional_connected: Connected 
    :type use_proportional_connected: bool
    :param use_proportional_projected: Projected (2D) 
    :type use_proportional_projected: bool
    :param release_confirm: Confirm on Release, Always confirm operation when releasing button 
    :type release_confirm: bool
    :param use_accurate: Accurate, Use accurate transformation 
    :type use_accurate: bool
    :param use_fill: Fill, Fill the ripped region 
    :type use_fill: bool
    '''

    pass


def rip_edge(mirror: bool = False,
             use_proportional_edit: bool = False,
             proportional_edit_falloff: typing.Union[int, str] = 'SMOOTH',
             proportional_size: float = 1.0,
             use_proportional_connected: bool = False,
             use_proportional_projected: bool = False,
             release_confirm: bool = False,
             use_accurate: bool = False):
    '''Extend vertices along the edge closest to the cursor 

    :param mirror: Mirror Editing 
    :type mirror: bool
    :param use_proportional_edit: Proportional Editing 
    :type use_proportional_edit: bool
    :param proportional_edit_falloff: Proportional Falloff, Falloff type for proportional editing modeSMOOTH Smooth, Smooth falloff.SPHERE Sphere, Spherical falloff.ROOT Root, Root falloff.INVERSE_SQUARE Inverse Square, Inverse Square falloff.SHARP Sharp, Sharp falloff.LINEAR Linear, Linear falloff.CONSTANT Constant, Constant falloff.RANDOM Random, Random falloff. 
    :type proportional_edit_falloff: typing.Union[int, str]
    :param proportional_size: Proportional Size 
    :type proportional_size: float
    :param use_proportional_connected: Connected 
    :type use_proportional_connected: bool
    :param use_proportional_projected: Projected (2D) 
    :type use_proportional_projected: bool
    :param release_confirm: Confirm on Release, Always confirm operation when releasing button 
    :type release_confirm: bool
    :param use_accurate: Accurate, Use accurate transformation 
    :type use_accurate: bool
    '''

    pass


def rip_edge_move(MESH_OT_rip_edge=None, TRANSFORM_OT_translate=None):
    '''Extend vertices and move the result 

    :param MESH_OT_rip_edge: Extend Vertices, Extend vertices along the edge closest to the cursor 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def rip_move(MESH_OT_rip=None, TRANSFORM_OT_translate=None):
    '''Rip polygons and move the result 

    :param MESH_OT_rip: Rip, Disconnect vertex or edges from connected geometry 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def screw(steps: int = 9,
          turns: int = 1,
          center: float = (0.0, 0.0, 0.0),
          axis: float = (0.0, 0.0, 0.0)):
    '''Extrude selected vertices in screw-shaped rotation around the cursor in indicated viewport 

    :param steps: Steps, Steps 
    :type steps: int
    :param turns: Turns, Turns 
    :type turns: int
    :param center: Center, Center in global view space 
    :type center: float
    :param axis: Axis, Axis in global view space 
    :type axis: float
    '''

    pass


def select_all(action: typing.Union[int, str] = 'TOGGLE'):
    '''(De)select all vertices, edges or faces 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_axis(orientation: typing.Union[int, str] = 'LOCAL',
                sign: typing.Union[int, str] = 'POS',
                axis: typing.Union[int, str] = 'X',
                threshold: float = 0.0001):
    '''Select all data in the mesh on a single axis 

    :param orientation: Axis Mode, Axis orientationGLOBAL Global, Align the transformation axes to world space.LOCAL Local, Align the transformation axes to the selected objects’ local space.NORMAL Normal, Align the transformation axes to average normal of selected elements (bone Y axis for pose mode).GIMBAL Gimbal, Align each axis to the Euler rotation axis as used for input.VIEW View, Align the transformation axes to the window.CURSOR Cursor, Align the transformation axes to the 3D cursor. 
    :type orientation: typing.Union[int, str]
    :param sign: Axis Sign, Side to select 
    :type sign: typing.Union[int, str]
    :param axis: Axis, Select the axis to compare each vertex on 
    :type axis: typing.Union[int, str]
    :param threshold: Threshold 
    :type threshold: float
    '''

    pass


def select_face_by_sides(number: int = 4,
                         type: typing.Union[int, str] = 'EQUAL',
                         extend: bool = True):
    '''Select vertices or faces by the number of polygon sides 

    :param number: Number of Vertices 
    :type number: int
    :param type: Type, Type of comparison to make 
    :type type: typing.Union[int, str]
    :param extend: Extend, Extend the selection 
    :type extend: bool
    '''

    pass


def select_interior_faces():
    '''Select faces where all edges have more than 2 face users 

    '''

    pass


def select_less(use_face_step: bool = True):
    '''Deselect vertices, edges or faces at the boundary of each selection region 

    :param use_face_step: Face Step, Connected faces (instead of edges) 
    :type use_face_step: bool
    '''

    pass


def select_linked(delimit: typing.Set[typing.Union[int, str]] = {'SEAM'}):
    '''Select all vertices connected to the current selection 

    :param delimit: Delimit, Delimit selected regionNORMAL Regular, Delimit by face directions.MATERIAL Material, Delimit by face material.SEAM Seam, Delimit by edge seams.SHARP Sharp, Delimit by sharp edges.UV UVs, Delimit by UV coordinates. 
    :type delimit: typing.Set[typing.Union[int, str]]
    '''

    pass


def select_linked_pick(deselect: bool = False,
                       delimit: typing.Set[typing.Union[int, str]] = {'SEAM'},
                       index=-1):
    '''(De)select all vertices linked to the edge under the mouse cursor 

    :param deselect: Deselect 
    :type deselect: bool
    :param delimit: Delimit, Delimit selected regionNORMAL Regular, Delimit by face directions.MATERIAL Material, Delimit by face material.SEAM Seam, Delimit by edge seams.SHARP Sharp, Delimit by sharp edges.UV UVs, Delimit by UV coordinates. 
    :type delimit: typing.Set[typing.Union[int, str]]
    '''

    pass


def select_loose(extend: bool = False):
    '''Select loose geometry based on the selection mode 

    :param extend: Extend, Extend the selection 
    :type extend: bool
    '''

    pass


def select_mirror(axis: typing.Set[typing.Union[int, str]] = {'X'},
                  extend: bool = False):
    '''Select mesh items at mirrored locations 

    :param axis: Axis 
    :type axis: typing.Set[typing.Union[int, str]]
    :param extend: Extend, Extend the existing selection 
    :type extend: bool
    '''

    pass


def select_mode(use_extend: bool = False,
                use_expand: bool = False,
                type: typing.Union[int, str] = 'VERT',
                action: typing.Union[int, str] = 'TOGGLE'):
    '''Change selection mode 

    :param use_extend: Extend 
    :type use_extend: bool
    :param use_expand: Expand 
    :type use_expand: bool
    :param type: Type 
    :type type: typing.Union[int, str]
    :param action: Action, Selection action to executeDISABLE Disable, Disable selected markers.ENABLE Enable, Enable selected markers.TOGGLE Toggle, Toggle disabled flag for selected markers. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_more(use_face_step: bool = True):
    '''Select more vertices, edges or faces connected to initial selection 

    :param use_face_step: Face Step, Connected faces (instead of edges) 
    :type use_face_step: bool
    '''

    pass


def select_next_item():
    '''Select the next element (using selection order) 

    '''

    pass


def select_non_manifold(extend: bool = True,
                        use_wire: bool = True,
                        use_boundary: bool = True,
                        use_multi_face: bool = True,
                        use_non_contiguous: bool = True,
                        use_verts: bool = True):
    '''Select all non-manifold vertices or edges 

    :param extend: Extend, Extend the selection 
    :type extend: bool
    :param use_wire: Wire, Wire edges 
    :type use_wire: bool
    :param use_boundary: Boundaries, Boundary edges 
    :type use_boundary: bool
    :param use_multi_face: Multiple Faces, Edges shared by 3+ faces 
    :type use_multi_face: bool
    :param use_non_contiguous: Non Contiguous, Edges between faces pointing in alternate directions 
    :type use_non_contiguous: bool
    :param use_verts: Vertices, Vertices connecting multiple face regions 
    :type use_verts: bool
    '''

    pass


def select_nth(nth: int = 2, skip: int = 1, offset: int = 0):
    '''Deselect every Nth element starting from the active vertex, edge or face 

    :param nth: Nth Element, Skip every Nth element 
    :type nth: int
    :param skip: Skip, Number of elements to skip at once 
    :type skip: int
    :param offset: Offset, Offset from the starting point 
    :type offset: int
    '''

    pass


def select_prev_item():
    '''Select the previous element (using selection order) 

    '''

    pass


def select_random(percent: float = 50.0,
                  seed: int = 0,
                  action: typing.Union[int, str] = 'SELECT'):
    '''Randomly select vertices 

    :param percent: Percent, Percentage of objects to select randomly 
    :type percent: float
    :param seed: Random Seed, Seed for the random number generator 
    :type seed: int
    :param action: Action, Selection action to executeSELECT Select, Select all elements.DESELECT Deselect, Deselect all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_similar(type: typing.Union[int, str] = 'NORMAL',
                   compare: typing.Union[int, str] = 'EQUAL',
                   threshold: float = 0.0):
    '''Select similar vertices, edges or faces by property types 

    :param type: Type 
    :type type: typing.Union[int, str]
    :param compare: Compare 
    :type compare: typing.Union[int, str]
    :param threshold: Threshold 
    :type threshold: float
    '''

    pass


def select_similar_region():
    '''Select similar face regions to the current selection 

    '''

    pass


def select_ungrouped(extend: bool = False):
    '''Select vertices without a group 

    :param extend: Extend, Extend the selection 
    :type extend: bool
    '''

    pass


def separate(type: typing.Union[int, str] = 'SELECTED'):
    '''Separate selected geometry into a new mesh 

    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def set_normals_from_faces(keep_sharp: bool = False):
    '''Set the custom normals from the selected faces ones 

    :param keep_sharp: Keep Sharp Edges, Do not set sharp edges to face 
    :type keep_sharp: bool
    '''

    pass


def shape_propagate_to_all():
    '''Apply selected vertex locations to all other shape keys 

    '''

    pass


def shortest_path_pick(edge_mode: typing.Union[int, str] = 'SELECT',
                       use_face_step: bool = False,
                       use_topology_distance: bool = False,
                       use_fill: bool = False,
                       nth: int = 1,
                       skip: int = 1,
                       offset: int = 0,
                       index=-1):
    '''Select shortest path between two selections 

    :param edge_mode: Edge Tag, The edge flag to tag when selecting the shortest path 
    :type edge_mode: typing.Union[int, str]
    :param use_face_step: Face Stepping, Traverse connected faces (includes diagonals and edge-rings) 
    :type use_face_step: bool
    :param use_topology_distance: Topology Distance, Find the minimum number of steps, ignoring spatial distance 
    :type use_topology_distance: bool
    :param use_fill: Fill Region, Select all paths between the source/destination elements 
    :type use_fill: bool
    :param nth: Nth Element, Skip every Nth element 
    :type nth: int
    :param skip: Skip, Number of elements to skip at once 
    :type skip: int
    :param offset: Offset, Offset from the starting point 
    :type offset: int
    '''

    pass


def shortest_path_select(edge_mode: typing.Union[int, str] = 'SELECT',
                         use_face_step: bool = False,
                         use_topology_distance: bool = False,
                         use_fill: bool = False,
                         nth: int = 1,
                         skip: int = 1,
                         offset: int = 0):
    '''Selected shortest path between two vertices/edges/faces 

    :param edge_mode: Edge Tag, The edge flag to tag when selecting the shortest path 
    :type edge_mode: typing.Union[int, str]
    :param use_face_step: Face Stepping, Traverse connected faces (includes diagonals and edge-rings) 
    :type use_face_step: bool
    :param use_topology_distance: Topology Distance, Find the minimum number of steps, ignoring spatial distance 
    :type use_topology_distance: bool
    :param use_fill: Fill Region, Select all paths between the source/destination elements 
    :type use_fill: bool
    :param nth: Nth Element, Skip every Nth element 
    :type nth: int
    :param skip: Skip, Number of elements to skip at once 
    :type skip: int
    :param offset: Offset, Offset from the starting point 
    :type offset: int
    '''

    pass


def smoothen_normals(factor: float = 0.5):
    '''Smoothen custom normals based on adjacent vertex normals 

    :param factor: Factor, Specifies weight of smooth vs original normal 
    :type factor: float
    '''

    pass


def solidify(thickness: float = 0.01):
    '''Create a solid skin by extruding, compensating for sharp angles 

    :param thickness: Thickness 
    :type thickness: float
    '''

    pass


def sort_elements(type: typing.Union[int, str] = 'VIEW_ZAXIS',
                  elements: typing.Set[typing.Union[int, str]] = {'VERT'},
                  reverse: bool = False,
                  seed: int = 0):
    '''The order of selected vertices/edges/faces is modified, based on a given method 

    :param type: Type, Type of re-ordering operation to applyVIEW_ZAXIS View Z Axis, Sort selected elements from farthest to nearest one in current view.VIEW_XAXIS View X Axis, Sort selected elements from left to right one in current view.CURSOR_DISTANCE Cursor Distance, Sort selected elements from nearest to farthest from 3D cursor.MATERIAL Material, Sort selected elements from smallest to greatest material index (faces only!).SELECTED Selected, Move all selected elements in first places, preserving their relative order (WARNING: this will affect unselected elements’ indices as well!).RANDOMIZE Randomize, Randomize order of selected elements.REVERSE Reverse, Reverse current order of selected elements. 
    :type type: typing.Union[int, str]
    :param elements: Elements, Which elements to affect (vertices, edges and/or faces) 
    :type elements: typing.Set[typing.Union[int, str]]
    :param reverse: Reverse, Reverse the sorting effect 
    :type reverse: bool
    :param seed: Seed, Seed for random-based operations 
    :type seed: int
    '''

    pass


def spin(steps: int = 9,
         dupli: bool = False,
         angle: float = 1.5708,
         use_auto_merge: bool = True,
         use_normal_flip: bool = False,
         center: float = (0.0, 0.0, 0.0),
         axis: float = (0.0, 0.0, 0.0)):
    '''Extrude selected vertices in a circle around the cursor in indicated viewport 

    :param steps: Steps, Steps 
    :type steps: int
    :param dupli: Duplicate, Make Duplicates 
    :type dupli: bool
    :param angle: Angle, Rotation for each step 
    :type angle: float
    :param use_auto_merge: Auto Merge, Merge first/last when the angle is a full revolution 
    :type use_auto_merge: bool
    :param use_normal_flip: Flip Normals 
    :type use_normal_flip: bool
    :param center: Center, Center in global view space 
    :type center: float
    :param axis: Axis, Axis in global view space 
    :type axis: float
    '''

    pass


def split():
    '''Split off selected geometry from connected unselected geometry 

    '''

    pass


def split_normals():
    '''Split custom normals of selected vertices 

    '''

    pass


def subdivide(number_cuts: int = 1,
              smoothness: float = 0.0,
              ngon: bool = True,
              quadcorner: typing.Union[int, str] = 'STRAIGHT_CUT',
              fractal: float = 0.0,
              fractal_along_normal: float = 0.0,
              seed: int = 0):
    '''Subdivide selected edges 

    :param number_cuts: Number of Cuts 
    :type number_cuts: int
    :param smoothness: Smoothness, Smoothness factor 
    :type smoothness: float
    :param ngon: Create N-Gons, When disabled, newly created faces are limited to 3-4 sided faces 
    :type ngon: bool
    :param quadcorner: Quad Corner Type, How to subdivide quad corners (anything other than Straight Cut will prevent ngons) 
    :type quadcorner: typing.Union[int, str]
    :param fractal: Fractal, Fractal randomness factor 
    :type fractal: float
    :param fractal_along_normal: Along Normal, Apply fractal displacement along normal only 
    :type fractal_along_normal: float
    :param seed: Random Seed, Seed for the random number generator 
    :type seed: int
    '''

    pass


def subdivide_edgering(number_cuts: int = 10,
                       interpolation: typing.Union[int, str] = 'PATH',
                       smoothness: float = 1.0,
                       profile_shape_factor: float = 0.0,
                       profile_shape: typing.Union[int, str] = 'SMOOTH'):
    '''Subdivide perpendicular edges to the selected edge ring 

    :param number_cuts: Number of Cuts 
    :type number_cuts: int
    :param interpolation: Interpolation, Interpolation method 
    :type interpolation: typing.Union[int, str]
    :param smoothness: Smoothness, Smoothness factor 
    :type smoothness: float
    :param profile_shape_factor: Profile Factor, How much intermediary new edges are shrunk/expanded 
    :type profile_shape_factor: float
    :param profile_shape: Profile Shape, Shape of the profileSMOOTH Smooth, Smooth falloff.SPHERE Sphere, Spherical falloff.ROOT Root, Root falloff.INVERSE_SQUARE Inverse Square, Inverse Square falloff.SHARP Sharp, Sharp falloff.LINEAR Linear, Linear falloff. 
    :type profile_shape: typing.Union[int, str]
    '''

    pass


def symmetrize(direction: typing.Union[int, str] = 'NEGATIVE_X',
               threshold: float = 0.0001):
    '''Enforce symmetry (both form and topological) across an axis 

    :param direction: Direction, Which sides to copy from and to 
    :type direction: typing.Union[int, str]
    :param threshold: Threshold, Limit for snap middle vertices to the axis center 
    :type threshold: float
    '''

    pass


def symmetry_snap(direction: typing.Union[int, str] = 'NEGATIVE_X',
                  threshold: float = 0.05,
                  factor: float = 0.5,
                  use_center: bool = True):
    '''Snap vertex pairs to their mirrored locations 

    :param direction: Direction, Which sides to copy from and to 
    :type direction: typing.Union[int, str]
    :param threshold: Threshold, Distance within which matching vertices are searched 
    :type threshold: float
    :param factor: Factor, Mix factor of the locations of the vertices 
    :type factor: float
    :param use_center: Center, Snap middle vertices to the axis center 
    :type use_center: bool
    '''

    pass


def tris_convert_to_quads(face_threshold: float = 0.698132,
                          shape_threshold: float = 0.698132,
                          uvs: bool = False,
                          vcols: bool = False,
                          seam: bool = False,
                          sharp: bool = False,
                          materials: bool = False):
    '''Join triangles into quads 

    :param face_threshold: Max Face Angle, Face angle limit 
    :type face_threshold: float
    :param shape_threshold: Max Shape Angle, Shape angle limit 
    :type shape_threshold: float
    :param uvs: Compare UVs 
    :type uvs: bool
    :param vcols: Compare VCols 
    :type vcols: bool
    :param seam: Compare Seam 
    :type seam: bool
    :param sharp: Compare Sharp 
    :type sharp: bool
    :param materials: Compare Materials 
    :type materials: bool
    '''

    pass


def unsubdivide(iterations: int = 2):
    '''UnSubdivide selected edges & faces 

    :param iterations: Iterations, Number of times to unsubdivide 
    :type iterations: int
    '''

    pass


def uv_texture_add():
    '''Add UV Map 

    '''

    pass


def uv_texture_remove():
    '''Remove UV Map 

    '''

    pass


def uvs_reverse():
    '''Flip direction of UV coordinates inside faces 

    '''

    pass


def uvs_rotate(use_ccw: bool = False):
    '''Rotate UV coordinates inside faces 

    :param use_ccw: Counter Clockwise 
    :type use_ccw: bool
    '''

    pass


def vert_connect():
    '''Connect selected vertices of faces, splitting the face 

    '''

    pass


def vert_connect_concave():
    '''Make all faces convex 

    '''

    pass


def vert_connect_nonplanar(angle_limit: float = 0.0872665):
    '''Split non-planar faces that exceed the angle threshold 

    :param angle_limit: Max Angle, Angle limit 
    :type angle_limit: float
    '''

    pass


def vert_connect_path():
    '''Connect vertices by their selection order, creating edges, splitting faces 

    '''

    pass


def vertex_color_add():
    '''Add vertex color layer 

    '''

    pass


def vertex_color_remove():
    '''Remove vertex color layer 

    '''

    pass


def vertices_smooth(factor: float = 0.5,
                    repeat: int = 1,
                    xaxis: bool = True,
                    yaxis: bool = True,
                    zaxis: bool = True):
    '''Flatten angles of selected vertices 

    :param factor: Smoothing, Smoothing factor 
    :type factor: float
    :param repeat: Repeat, Number of times to smooth the mesh 
    :type repeat: int
    :param xaxis: X-Axis, Smooth along the X axis 
    :type xaxis: bool
    :param yaxis: Y-Axis, Smooth along the Y axis 
    :type yaxis: bool
    :param zaxis: Z-Axis, Smooth along the Z axis 
    :type zaxis: bool
    '''

    pass


def vertices_smooth_laplacian(repeat: int = 1,
                              lambda_factor: float = 1.0,
                              lambda_border: float = 5e-05,
                              use_x: bool = True,
                              use_y: bool = True,
                              use_z: bool = True,
                              preserve_volume: bool = True):
    '''Laplacian smooth of selected vertices 

    :param repeat: Number of iterations to smooth the mesh 
    :type repeat: int
    :param lambda_factor: Lambda factor 
    :type lambda_factor: float
    :param lambda_border: Lambda factor in border 
    :type lambda_border: float
    :param use_x: Smooth X Axis, Smooth object along X axis 
    :type use_x: bool
    :param use_y: Smooth Y Axis, Smooth object along Y axis 
    :type use_y: bool
    :param use_z: Smooth Z Axis, Smooth object along Z axis 
    :type use_z: bool
    :param preserve_volume: Preserve Volume, Apply volume preservation after smooth 
    :type preserve_volume: bool
    '''

    pass


def wireframe(use_boundary: bool = True,
              use_even_offset: bool = True,
              use_relative_offset: bool = False,
              use_replace: bool = True,
              thickness: float = 0.01,
              offset: float = 0.01,
              use_crease: bool = False,
              crease_weight: float = 0.01):
    '''Create a solid wire-frame from faces 

    :param use_boundary: Boundary, Inset face boundaries 
    :type use_boundary: bool
    :param use_even_offset: Offset Even, Scale the offset to give more even thickness 
    :type use_even_offset: bool
    :param use_relative_offset: Offset Relative, Scale the offset by surrounding geometry 
    :type use_relative_offset: bool
    :param use_replace: Replace, Remove original faces 
    :type use_replace: bool
    :param thickness: Thickness 
    :type thickness: float
    :param offset: Offset 
    :type offset: float
    :param use_crease: Crease, Crease hub edges for improved subsurf 
    :type use_crease: bool
    :param crease_weight: Crease weight 
    :type crease_weight: float
    '''

    pass



import sys
import typing


def bvh(filepath: str = "",
        check_existing: bool = True,
        filter_glob: str = "*.bvh",
        global_scale: float = 1.0,
        frame_start: int = 0,
        frame_end: int = 0,
        rotate_mode: typing.Union[int, str] = 'NATIVE',
        root_transform_only: bool = False):
    '''Save a BVH motion capture file from an armature 

    :param filepath: File Path, Filepath used for exporting the file 
    :type filepath: str
    :param check_existing: Check Existing, Check and warn on overwriting existing files 
    :type check_existing: bool
    :param filter_glob: filter_glob 
    :type filter_glob: str
    :param global_scale: Scale, Scale the BVH by this value 
    :type global_scale: float
    :param frame_start: Start Frame, Starting frame to export 
    :type frame_start: int
    :param frame_end: End Frame, End frame to export 
    :type frame_end: int
    :param rotate_mode: Rotation, Rotation conversionNATIVE Euler (Native), Use the rotation order defined in the BVH file.XYZ Euler (XYZ), Convert rotations to euler XYZ.XZY Euler (XZY), Convert rotations to euler XZY.YXZ Euler (YXZ), Convert rotations to euler YXZ.YZX Euler (YZX), Convert rotations to euler YZX.ZXY Euler (ZXY), Convert rotations to euler ZXY.ZYX Euler (ZYX), Convert rotations to euler ZYX. 
    :type rotate_mode: typing.Union[int, str]
    :param root_transform_only: Root Translation Only, Only write out translation channels for the root bone 
    :type root_transform_only: bool
    '''

    pass



import sys
import typing


class Device:
    '''Unlocks the device after a lock call, see lock() for details. '''

    channels = None
    '''The channel count of the device. '''

    distance_model = None
    '''The distance model of the device. '''

    doppler_factor = None
    '''The doppler factor of the device. This factor is a scaling factor for the velocity vectors in doppler calculation. So a value bigger than 1 will exaggerate the effect as it raises the velocity. '''

    format = None
    '''The native sample format of the device. '''

    listener_location = None
    '''The listeners’s location in 3D space, a 3D tuple of floats. '''

    listener_orientation = None
    '''The listener’s orientation in 3D space as quaternion, a 4 float tuple. '''

    listener_velocity = None
    '''The listener’s velocity in 3D space, a 3D tuple of floats. '''

    rate = None
    '''The sampling rate of the device in Hz. '''

    speed_of_sound = None
    '''The speed of sound of the device. The speed of sound in air is typically 343.3 m/s. '''

    volume = None
    '''The overall volume of the device. '''


class DynamicMusic:
    '''Stops playback of the scene. '''

    fadeTime = None
    '''The length in seconds of the crossfade transition '''

    position = None
    '''The playback position of the scene in seconds. '''

    scene = None
    '''The current scene '''

    status = None
    '''Whether the scene is playing, paused or stopped (=invalid). '''

    volume = None
    '''The volume of the scene. '''


class Handle:
    '''Stops playback. '''

    attenuation = None
    '''This factor is used for distance based attenuation of the source. '''

    cone_angle_inner = None
    '''The opening angle of the inner cone of the source. If the cone values of a source are set there are two (audible) cones with the apex at the location of the source and with infinite height, heading in the direction of the source’s orientation. In the inner cone the volume is normal. Outside the outer cone the volume will be cone_volume_outer and in the area between the volume will be interpolated linearly. '''

    cone_angle_outer = None
    '''The opening angle of the outer cone of the source. '''

    cone_volume_outer = None
    '''The volume outside the outer cone of the source. '''

    distance_maximum = None
    '''The maximum distance of the source. If the listener is further away the source volume will be 0. '''

    distance_reference = None
    '''The reference distance of the source. At this distance the volume will be exactly volume. '''

    keep = None
    '''Whether the sound should be kept paused in the device when its end is reached. This can be used to seek the sound to some position and start playback again. '''

    location = None
    '''The source’s location in 3D space, a 3D tuple of floats. '''

    loop_count = None
    '''The (remaining) loop count of the sound. A negative value indicates infinity. '''

    orientation = None
    '''The source’s orientation in 3D space as quaternion, a 4 float tuple. '''

    pitch = None
    '''The pitch of the sound. '''

    position = None
    '''The playback position of the sound in seconds. '''

    relative = None
    '''Whether the source’s location, velocity and orientation is relative or absolute to the listener. '''

    status = None
    '''Whether the sound is playing, paused or stopped (=invalid). '''

    velocity = None
    '''The source’s velocity in 3D space, a 3D tuple of floats. '''

    volume = None
    '''The volume of the sound. '''

    volume_maximum = None
    '''The maximum volume of the source. '''

    volume_minimum = None
    '''The minimum volume of the source. '''


class PlaybackManager:
    '''Stops playback of the category. '''

    pass


class Sequence:
    '''Writes animation data to a sequence. '''

    channels = None
    '''The channel count of the sequence. '''

    distance_model = None
    '''The distance model of the sequence. '''

    doppler_factor = None
    '''The doppler factor of the sequence. This factor is a scaling factor for the velocity vectors in doppler calculation. So a value bigger than 1 will exaggerate the effect as it raises the velocity. '''

    fps = None
    '''The listeners’s location in 3D space, a 3D tuple of floats. '''

    muted = None
    '''Whether the whole sequence is muted. '''

    rate = None
    '''The sampling rate of the sequence in Hz. '''

    speed_of_sound = None
    '''The speed of sound of the sequence. The speed of sound in air is typically 343.3 m/s. '''


class SequenceEntry:
    '''Writes animation data to a sequenced entry. '''

    attenuation = None
    '''This factor is used for distance based attenuation of the source. '''

    cone_angle_inner = None
    '''The opening angle of the inner cone of the source. If the cone values of a source are set there are two (audible) cones with the apex at the location of the source and with infinite height, heading in the direction of the source’s orientation. In the inner cone the volume is normal. Outside the outer cone the volume will be cone_volume_outer and in the area between the volume will be interpolated linearly. '''

    cone_angle_outer = None
    '''The opening angle of the outer cone of the source. '''

    cone_volume_outer = None
    '''The volume outside the outer cone of the source. '''

    distance_maximum = None
    '''The maximum distance of the source. If the listener is further away the source volume will be 0. '''

    distance_reference = None
    '''The reference distance of the source. At this distance the volume will be exactly volume. '''

    muted = None
    '''Whether the entry is muted. '''

    relative = None
    '''Whether the source’s location, velocity and orientation is relative or absolute to the listener. '''

    sound = None
    '''The sound the entry is representing and will be played in the sequence. '''

    volume_maximum = None
    '''The maximum volume of the source. '''

    volume_minimum = None
    '''The minimum volume of the source. '''


class Sound:
    '''Writes the sound to a file. '''

    length = None
    '''The sample specification of the sound as a tuple with rate and channel count. '''

    specs = None
    '''The sample specification of the sound as a tuple with rate and channel count. '''


class Source:
    '''The source object represents the source position of a binaural sound. '''

    azimuth = None
    '''The azimuth angle. '''

    distance = None
    '''The distance value. 0 is min, 1 is max. '''

    elevation = None
    '''The elevation angle. '''


class ThreadPool:
    '''A ThreadPool is used to parallelize convolution efficiently. '''

    pass


class error:
    pass


AP_LOCATION = None
'''constant value 3 '''

AP_ORIENTATION = None
'''constant value 4 '''

AP_PANNING = None
'''constant value 1 '''

AP_PITCH = None
'''constant value 2 '''

AP_VOLUME = None
'''constant value 0 '''

CHANNELS_INVALID = None
'''constant value 0 '''

CHANNELS_MONO = None
'''constant value 1 '''

CHANNELS_STEREO = None
'''constant value 2 '''

CHANNELS_STEREO_LFE = None
'''constant value 3 '''

CHANNELS_SURROUND4 = None
'''constant value 4 '''

CHANNELS_SURROUND5 = None
'''constant value 5 '''

CHANNELS_SURROUND51 = None
'''constant value 6 '''

CHANNELS_SURROUND61 = None
'''constant value 7 '''

CHANNELS_SURROUND71 = None
'''constant value 8 '''

CODEC_AAC = None
'''constant value 1 '''

CODEC_AC3 = None
'''constant value 2 '''

CODEC_FLAC = None
'''constant value 3 '''

CODEC_INVALID = None
'''constant value 0 '''

CODEC_MP2 = None
'''constant value 4 '''

CODEC_MP3 = None
'''constant value 5 '''

CODEC_OPUS = None
'''constant value 8 '''

CODEC_PCM = None
'''constant value 6 '''

CODEC_VORBIS = None
'''constant value 7 '''

CONTAINER_AC3 = None
'''constant value 1 '''

CONTAINER_FLAC = None
'''constant value 2 '''

CONTAINER_INVALID = None
'''constant value 0 '''

CONTAINER_MATROSKA = None
'''constant value 3 '''

CONTAINER_MP2 = None
'''constant value 4 '''

CONTAINER_MP3 = None
'''constant value 5 '''

CONTAINER_OGG = None
'''constant value 6 '''

CONTAINER_WAV = None
'''constant value 7 '''

DISTANCE_MODEL_EXPONENT = None
'''constant value 5 '''

DISTANCE_MODEL_EXPONENT_CLAMPED = None
'''constant value 6 '''

DISTANCE_MODEL_INVALID = None
'''constant value 0 '''

DISTANCE_MODEL_INVERSE = None
'''constant value 1 '''

DISTANCE_MODEL_INVERSE_CLAMPED = None
'''constant value 2 '''

DISTANCE_MODEL_LINEAR = None
'''constant value 3 '''

DISTANCE_MODEL_LINEAR_CLAMPED = None
'''constant value 4 '''

FORMAT_FLOAT32 = None
'''constant value 36 '''

FORMAT_FLOAT64 = None
'''constant value 40 '''

FORMAT_INVALID = None
'''constant value 0 '''

FORMAT_S16 = None
'''constant value 18 '''

FORMAT_S24 = None
'''constant value 19 '''

FORMAT_S32 = None
'''constant value 20 '''

FORMAT_U8 = None
'''constant value 1 '''

RATE_11025 = None
'''constant value 11025 '''

RATE_16000 = None
'''constant value 16000 '''

RATE_192000 = None
'''constant value 192000 '''

RATE_22050 = None
'''constant value 22050 '''

RATE_32000 = None
'''constant value 32000 '''

RATE_44100 = None
'''constant value 44100 '''

RATE_48000 = None
'''constant value 48000 '''

RATE_8000 = None
'''constant value 8000 '''

RATE_88200 = None
'''constant value 88200 '''

RATE_96000 = None
'''constant value 96000 '''

RATE_INVALID = None
'''constant value 0 '''

STATUS_INVALID = None
'''constant value 0 '''

STATUS_PAUSED = None
'''constant value 2 '''

STATUS_PLAYING = None
'''constant value 1 '''

STATUS_STOPPED = None
'''constant value 3 '''
